//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file        CallManager.cpp
 * @brief       This is the implementation file for CallManager class.
 */

#include <FTelCallManager.h>
#include <FBaseSysLog.h>
#include <FSec_AccessController.h>
#include <FSys_SystemInfoImpl.h>
#include "FTel_CallManagerImpl.h"

using namespace Tizen::Base;
using namespace Tizen::Security;
using namespace Tizen::System;


namespace Tizen { namespace Telephony
{
CallManager::CallManager(void)
	: __pCallManagerImpl(null)
{
}

CallManager::~CallManager(void)
{
	delete __pCallManagerImpl;
}

result
CallManager::Construct(void)
{
	static const wchar_t _TELEPHONY[] = L"http://tizen.org/feature/network.telephony";

	SysAssertf(__pCallManagerImpl  == null,
	                "Already constructed. Calling Construct() twice or more on a same instance is not allowed for this class.");

	result r = E_SUCCESS;
	bool isTelephonySupported = false;

    r = _SystemInfoImpl::GetSysInfo(_TELEPHONY, isTelephonySupported);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS && isTelephonySupported, E_UNSUPPORTED_OPERATION, "Telephony is not supported.");

	__pCallManagerImpl = new (std::nothrow) _CallManagerImpl();
	SysTryReturnResult(NID_TEL, __pCallManagerImpl != null, E_OUT_OF_MEMORY, "Memory allocation failed.");

	r = __pCallManagerImpl->Construct();
	if (r != E_SUCCESS)
	{
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
		delete __pCallManagerImpl;
		__pCallManagerImpl = null;
	}

	return r;
}

result
CallManager::Construct(ITelephonyCallEventListener& listener)
{
	static const wchar_t _TELEPHONY[] = L"http://tizen.org/feature/network.telephony";

	SysAssertf(__pCallManagerImpl == null,
	        "Already constructed. Calling Construct() twice or more on a same instance is not allowed for this class.");

	result r = E_SUCCESS;
	bool isTelephonySupported = false;

    r = _SystemInfoImpl::GetSysInfo(_TELEPHONY, isTelephonySupported);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS && isTelephonySupported, E_UNSUPPORTED_OPERATION, "Telephony is not supported.");

	__pCallManagerImpl = new (std::nothrow) _CallManagerImpl();
	SysTryReturnResult(NID_TEL, __pCallManagerImpl != null, E_OUT_OF_MEMORY, "Memory allocation failed.");

	r = __pCallManagerImpl->Construct(&listener);

	if (r != E_SUCCESS)
	{
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
		delete __pCallManagerImpl;
		__pCallManagerImpl = null;
	}

	return r;
}

result
CallManager::SetCallForwardListener(ITelephonyCallForwardListener* pListener)
{
	SysAssertf(__pCallManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = __pCallManagerImpl->SetCallForwardListener(pListener);

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

	return r;
}

result
CallManager::RequestCallForward(const String& phoneNumber)
{
	SysAssertf(__pCallManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = __pCallManagerImpl->RequestCallForward(phoneNumber);

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

	return r;
}

result
CallManager::StopCallForward(void)
{
	SysAssertf(__pCallManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = __pCallManagerImpl->StopCallForward();

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

	return r;
}

result
CallManager::GetCallForwardNumber(void) const
{
	SysAssertf(__pCallManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = __pCallManagerImpl->GetCallForwardNumber();

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

	return r;
}

CallType
CallManager::GetCurrentCallType(void) const
{
	SysAssertf(__pCallManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	CallType callType = TYPE_UNDEFINED_CALL;

	callType = __pCallManagerImpl->GetCurrentCallType();

	return callType;
}

CallStatus
CallManager::GetCurrentCallStatus(void) const
{
	SysAssertf(__pCallManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	CallStatus callStatus = CALL_STATUS_UNDEFINED;

	callStatus = __pCallManagerImpl->GetCurrentCallStatus();

	return callStatus;
}

}} // Tizen::Telephony
