//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file        FTelNetworkInfo.cpp
 * @brief       This is the implementation file for NetworkInfo class.
 */

#include <FTelNetworkInfo.h>
#include <FBaseSysLog.h>
#include <FSec_AccessController.h>
#include <FSys_SystemInfoImpl.h>
#include "FTel_NetworkInfoImpl.h"


using namespace Tizen::Base;
using namespace Tizen::Security;
using namespace Tizen::System;


namespace Tizen { namespace Telephony
{

NetworkInfo::NetworkInfo(void)
	: __pNetworkInfoImpl(null)
{
	__pNetworkInfoImpl = new (std::nothrow) _NetworkInfoImpl();
	SysTryReturnVoidResult(NID_TEL, __pNetworkInfoImpl != null, E_OUT_OF_MEMORY, "Memory allocation failed.");
}

NetworkInfo::NetworkInfo(const NetworkInfo& value)
{
	__pNetworkInfoImpl = new (std::nothrow) _NetworkInfoImpl(*(value.__pNetworkInfoImpl));
	SysTryReturnVoidResult(NID_TEL, __pNetworkInfoImpl != null, E_OUT_OF_MEMORY, "Memory allocation failed.");
}

NetworkInfo::~NetworkInfo(void)
{
	delete __pNetworkInfoImpl;
}

NetworkInfo&
NetworkInfo::operator=(const NetworkInfo& rhs)
{
	if (this != &rhs)
    {
    	*__pNetworkInfoImpl = *(rhs.__pNetworkInfoImpl);
	}

	return *this;
}

int
NetworkInfo::GetMnc(void) const
{
	ClearLastResult();

	int mnc = -1;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, mnc, E_PRIVILEGE_DENIED,
            "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	mnc = __pNetworkInfoImpl->GetMnc();

	return mnc;
}

int
NetworkInfo::GetMcc(void) const
{
	ClearLastResult();

	int mcc = -1;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, mcc, E_PRIVILEGE_DENIED,
	            "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	mcc = __pNetworkInfoImpl->GetMcc();

	return mcc;
}

String
NetworkInfo::GetPlmn(void) const
{
	ClearLastResult();

	String plmn;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, plmn, E_PRIVILEGE_DENIED,
	            "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	plmn = __pNetworkInfoImpl->GetPlmn();

	return plmn;
}

int
NetworkInfo::GetCellId(void) const
{
	ClearLastResult();

	int cellId = -1;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, cellId, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	cellId = __pNetworkInfoImpl->GetCellId();

	return cellId;
}

int
NetworkInfo::GetLac(void) const
{
	ClearLastResult();

	int lac = -1;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, lac, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	lac = __pNetworkInfoImpl->GetLac();

	return lac;
}

String
NetworkInfo::GetOperatorName(void) const
{
    ClearLastResult();

    String operatorName;
    result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

    SysTryReturn(NID_TEL, r == E_SUCCESS, operatorName, E_PRIVILEGE_DENIED,
                "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

    operatorName = __pNetworkInfoImpl->GetOperatorName();

    return operatorName;
}

bool
NetworkInfo::Equals(const Object& obj) const
{
	const NetworkInfo* pOther = dynamic_cast<const NetworkInfo*>(&obj);

	if (pOther == null)
	{
    	return false;
	}

	if (pOther == this)
	{
    	return true;
	}

	return __pNetworkInfoImpl->Equals(*pOther->__pNetworkInfoImpl);
}

int
NetworkInfo::GetHashCode(void) const
{
	return __pNetworkInfoImpl->GetHashCode();
}

int
NetworkInfo::GetRssi(void)
{
	static const wchar_t _TELEPHONY[] = L"http://tizen.org/feature/network.telephony";

	ClearLastResult();

	int rssi = -1;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);
	SysTryReturn(NID_TEL, r == E_SUCCESS, rssi, E_PRIVILEGE_DENIED,
			"[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	bool isTelephonySupported = false;

    r = _SystemInfoImpl::GetSysInfo(_TELEPHONY, isTelephonySupported);
    SysTryReturn(NID_TEL, r == E_SUCCESS && isTelephonySupported, rssi, E_UNSUPPORTED_OPERATION,
    		"[%s] Telephony is not supported.", GetErrorMessage(E_UNSUPPORTED_OPERATION));

	rssi = _NetworkInfoImpl::GetRssi();

	r = GetLastResult();

	SysTryReturn(NID_TEL, r == E_SUCCESS, rssi, r, "[%s] Propagating.", GetErrorMessage(r));

	return rssi;
}

}} // Tizen::Telephony
