//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file        FTelNetworkManager.cpp
 * @brief       This is the implementation file for NetworkManager class.
 */

#include <FTelNetworkManager.h>
#include <FBaseSysLog.h>
#include <FTelITelephonyNetworkEventListener.h>
#include <FTelITelephonyNetworkSettingListener.h>
#include <FSec_AccessController.h>
#include <FSys_SystemInfoImpl.h>
#include "FTel_NetworkManagerImpl.h"

using namespace Tizen::Security;
using namespace Tizen::System;

namespace Tizen { namespace Telephony
{
NetworkManager::NetworkManager(void)
	: __pNetworkManagerImpl(null)
{

}

NetworkManager::~NetworkManager(void)
{
	delete __pNetworkManagerImpl;
}

result
NetworkManager::Construct(ITelephonyNetworkEventListener* pListener)
{
	static const wchar_t _TELEPHONY[] = L"http://tizen.org/feature/network.telephony";

	SysAssertf(__pNetworkManagerImpl == null,
	        "Already constructed. Calling Construct() twice or more on a same instance is not allowed for this class.");

	result r = E_SUCCESS;
	bool isTelephonySupported = false;

    r = _SystemInfoImpl::GetSysInfo(_TELEPHONY, isTelephonySupported);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS && isTelephonySupported, E_UNSUPPORTED_OPERATION, "Telephony is not supported.");

	__pNetworkManagerImpl = new (std::nothrow) _NetworkManagerImpl();
	SysTryReturnResult(NID_TEL, __pNetworkManagerImpl != null, E_OUT_OF_MEMORY, "Memory allocation failed.");

	r = __pNetworkManagerImpl->Construct(pListener);

	if (r != E_SUCCESS)
	{
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
		delete __pNetworkManagerImpl;
		__pNetworkManagerImpl = null;
	}

	return r;
}

result
NetworkManager::GetNetworkStatus(NetworkStatus& networkStatus) const
{
	SysAssertf(__pNetworkManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_PRIVILEGE_DENIED, "The application does not have the privilege to call this method.");

	r = __pNetworkManagerImpl->GetNetworkStatus(networkStatus);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	return r;
}

result
NetworkManager::GetNetworkInfo(NetworkInfo& networkInfo) const
{
	SysAssertf(__pNetworkManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_PRIVILEGE_DENIED, "The application does not have the privilege to call this method.");


	r = __pNetworkManagerImpl->GetNetworkInfo(networkInfo);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	return r;
}



result
NetworkManager::SetNetworkSettingListener(ITelephonyNetworkSettingListener* pListener)
{
    SysAssertf(__pNetworkManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

    result r = __pNetworkManagerImpl->SetNetworkSettingListener(pListener);

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

    return r;
}

result
NetworkManager::GetNetworkSelectionMode(void)
{
    SysAssertf(__pNetworkManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

    result r = __pNetworkManagerImpl->GetNetworkSelectionMode();

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

    return r;

}

result
NetworkManager::SelectNetwork(const NetworkInfo& networkInfo)
{
    SysAssertf(__pNetworkManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

    result r = __pNetworkManagerImpl->SelectNetwork(networkInfo);

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

    return r;

}

result
NetworkManager::SelectNetwork(void)
{
    SysAssertf(__pNetworkManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

    result r = __pNetworkManagerImpl->SelectNetwork();

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

    return r;

}

result
NetworkManager::SearchNetwork(void)
{
    SysAssertf(__pNetworkManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

    result r = __pNetworkManagerImpl->SearchNetwork();

    if (r == E_PRIVILEGE_DENIED)
    {
        SysLogException(NID_TEL, E_PRIVILEGE_DENIED, "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
    }
    else if( r != E_SUCCESS)
    {
        SysLogException(NID_TEL, r,"[%s] Propagating.", GetErrorMessage(r));
    }

    return r;
}

}} // Tizen::Telephony
