//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file        FTelSimInfo.cpp
 * @brief       This is the implementation file for SimInfo class.
 */

#include <FTelSimInfo.h>
#include <FBaseSysLog.h>
#include <FSec_AccessController.h>
#include <FSys_SystemInfoImpl.h>
#include "FTel_SimInfoImpl.h"

using namespace Tizen::Base;
using namespace Tizen::Security;
using namespace Tizen::System;

namespace Tizen { namespace Telephony
{
SimInfo::SimInfo(void)
	: __pSimInfoImpl(null)
{
}

SimInfo::SimInfo(const SimInfo& rhs)
	: __pSimInfoImpl(null)
{
	__pSimInfoImpl = new (std::nothrow)_SimInfoImpl(*rhs.__pSimInfoImpl);
	SysTryReturnVoidResult(NID_TEL, __pSimInfoImpl, E_OUT_OF_MEMORY, "[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));
}

SimInfo::~SimInfo(void)
{
	delete __pSimInfoImpl;
}


result
SimInfo::Construct(void)
{
	static const wchar_t _TELEPHONY[] = L"http://tizen.org/feature/network.telephony";

	SysAssertf(__pSimInfoImpl == null,
	        "Already constructed. Calling Construct() twice or more on a same instance is not allowed for this class.");

	result r = E_SUCCESS;
	bool isTelephonySupported = false;

    r = _SystemInfoImpl::GetSysInfo(_TELEPHONY, isTelephonySupported);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS && isTelephonySupported, E_UNSUPPORTED_OPERATION, "Telephony is not supported.");

	__pSimInfoImpl = new (std::nothrow)_SimInfoImpl();
	SysTryReturnResult(NID_TEL, __pSimInfoImpl != null, E_OUT_OF_MEMORY, "Memory allocation failed");

	r = __pSimInfoImpl->Construct();

	if (r != E_SUCCESS)
	{
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
		delete __pSimInfoImpl;
		__pSimInfoImpl = null;
	}

	return r;
}

int
SimInfo::GetMnc(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	int mnc = -1;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, mnc, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	mnc = __pSimInfoImpl->GetMnc();

	r = GetLastResult();

	SysTryReturn(NID_TEL, r == E_SUCCESS, mnc, r, "[%s] Propagating.", GetErrorMessage(r));

	return mnc;
}

int
SimInfo::GetMcc(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	int mcc = -1;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, mcc, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	mcc = __pSimInfoImpl->GetMcc();

	r = GetLastResult();

	SysTryReturn(NID_TEL, r == E_SUCCESS, mcc, r, "[%s] Propagating.", GetErrorMessage(r));

	return mcc;
}

String
SimInfo::GetSpn(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	String spn;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, spn, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	spn = __pSimInfoImpl->GetSpn();

	r = GetLastResult();

	SysTryReturn(NID_TEL, r == E_SUCCESS, spn, r, "[%s] Propagating.", GetErrorMessage(r));

	return spn;
}

String
SimInfo::GetIccId(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	String iccId;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, iccId, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	iccId = __pSimInfoImpl->GetIccId();

	r = GetLastResult();

	SysTryReturn(NID_TEL, r == E_SUCCESS, iccId, r, "[%s] Propagating.", GetErrorMessage(r));

	return iccId;
}

String
SimInfo::GetOperatorName(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	String operatorName;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, operatorName, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	operatorName = __pSimInfoImpl->GetOperatorName();

	r = GetLastResult();

	SysTryReturn(NID_TEL, r == E_SUCCESS, operatorName, r, "[%s] Propagating.", GetErrorMessage(r));

	return operatorName;
}

String
SimInfo::GetPhoneNumber(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	String phoneNumber;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, phoneNumber, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	phoneNumber = __pSimInfoImpl->GetPhoneNumber();

	r = GetLastResult();

	SysTryReturn(NID_TEL, r == E_SUCCESS, phoneNumber, r, "[%s] Propagating.", GetErrorMessage(r));

	return phoneNumber;
}

String
SimInfo::GetImsi(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	String imsi;

	imsi = __pSimInfoImpl->GetImsi();

	result r = GetLastResult();

	if (r == E_PRIVILEGE_DENIED)
	{
		SysLogException(NID_TEL, E_PRIVILEGE_DENIED,"[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));
	}
	else if(r != E_SUCCESS)
	{
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
	}

	return imsi;
}

SimType
SimInfo::GetSimType(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	result r = _AccessController::CheckUserPrivilege(_PRV_TELEPHONY);
	SysTryReturn(NID_TEL, r == E_SUCCESS, SIM_TYPE_UNKNOWN, E_PRIVILEGE_DENIED,
			"[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	return __pSimInfoImpl->GetSimType();
}

bool
SimInfo::IsAvailable(void) const
{
	SysAssertf(__pSimInfoImpl != null, "Not yet constructed. Construct() should be called before use.");

	ClearLastResult();

	bool isAvailable = false;
	result r = _AccessController::CheckUserPrivilege(_PRV_SYSTEMINFO, _PRV_TELEPHONY);

	SysTryReturn(NID_TEL, r == E_SUCCESS, isAvailable, E_PRIVILEGE_DENIED,
	        "[%s] The application does not have the privilege to call this method.", GetErrorMessage(E_PRIVILEGE_DENIED));

	isAvailable = __pSimInfoImpl->IsAvailable();

	return isAvailable;
}

SimInfo&
SimInfo::operator =(const SimInfo& rhs)
{
    if (this != &rhs)
    {
    	if (rhs.__pSimInfoImpl != null)
    	{
    		if (__pSimInfoImpl != null)
    		{
				*__pSimInfoImpl = *rhs.__pSimInfoImpl;
    		}
    		else
    		{
    			__pSimInfoImpl = new (std::nothrow) _SimInfoImpl(*rhs.__pSimInfoImpl);
    		}
    	}
    	else
    	{
    		delete __pSimInfoImpl;
    		__pSimInfoImpl = null;
    	}
    }

    return *this;
}

bool
SimInfo::Equals(const Object& rhs) const
{
	const SimInfo* pRhs = dynamic_cast<const SimInfo*>(&rhs);

	if (pRhs == null)
	{
		return false;
	}

	const _SimInfoImpl* pRhsImpl = _SimInfoImpl::GetInstance(*pRhs);

	if (pRhsImpl == null)
	{
		return false;
	}

	return __pSimInfoImpl->Equals(*pRhsImpl);
}

int
SimInfo::GetHashCode(void) const
{
	return __pSimInfoImpl->GetHashCode();
}

} } // Tizen::Telephony
