//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file        FTelSimStateManager.cpp
 * @brief       This is the implementation file for SimStateManager class.
 */

#include <FBaseSysLog.h>
#include <FSec_AccessController.h>
#include <FTelSimStateManager.h>
#include <FSys_SystemInfoImpl.h>
#include "FTel_SimStateManagerImpl.h"


using namespace Tizen::Base;
using namespace Tizen::Security;
using namespace Tizen::System;


namespace Tizen { namespace Telephony
{
SimStateManager::SimStateManager(void)
	: __pSimStateManagerImpl(null)
{
}

SimStateManager::~SimStateManager(void)
{
	delete __pSimStateManagerImpl;
}

result
SimStateManager::Construct(void)
{
	static const wchar_t _TELEPHONY[] = L"http://tizen.org/feature/network.telephony";

	SysAssertf(__pSimStateManagerImpl == null,
	        "Already constructed. Calling Construct() twice or more on a same instance is not allowed for this class.");

	result r = E_SUCCESS;
	bool isTelephonySupported = false;

    r = _SystemInfoImpl::GetSysInfo(_TELEPHONY, isTelephonySupported);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS && isTelephonySupported, E_UNSUPPORTED_OPERATION, "Telephony is not supported.");

	__pSimStateManagerImpl = new (std::nothrow)_SimStateManagerImpl();
	SysTryReturnResult(NID_TEL, __pSimStateManagerImpl != null, E_OUT_OF_MEMORY, "Memory allocation failed");

	r = __pSimStateManagerImpl->Construct();

	if (r != E_SUCCESS)
	{
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
		delete __pSimStateManagerImpl;
		__pSimStateManagerImpl = null;
	}

	return r;
}

result
SimStateManager::SetSimEventListener(ITelephonySimEventListener* pListener)
{
	SysAssertf(__pSimStateManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = _AccessController::CheckUserPrivilege(_PRV_TELEPHONY);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "The application does not have the privilege to call this method.");

	r = __pSimStateManagerImpl->SetSimEventListener(pListener);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	return r;
}

result
SimStateManager::GetPinLockSetting(ISimStateManagerGetPinLockSettingResultListener* pListener)
{
	SysAssertf(__pSimStateManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = _AccessController::CheckUserPrivilege(_PRV_TELEPHONY);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "The application does not have the privilege to call this method.");

	r = __pSimStateManagerImpl->GetPinLockSetting(pListener);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	return r;
}

SimState
SimStateManager::GetSimState(void) const
{
	SysAssertf(__pSimStateManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = _AccessController::CheckUserPrivilege(_PRV_TELEPHONY);
	SysTryReturn(NID_TEL, r == E_SUCCESS, SIM_STATE_UNKNOWN, r,
			"[%s] The application does not have the privilege to call this method.", GetErrorMessage(r));

	return __pSimStateManagerImpl->GetSimState();
}

result
SimStateManager::GetSimInfo(SimInfo& simInfo) const
{
	SysAssertf(__pSimStateManagerImpl != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	r = _AccessController::CheckUserPrivilege(_PRV_TELEPHONY);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "The application does not have the privilege to call this method.");

	r = __pSimStateManagerImpl->GetSimInfo(simInfo);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	return r;
}

} } // Tizen::Telephony
