//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file    FTel_CallManagerImpl.cpp
 * @brief   This is the implementation file for _CallManagerImpl class.
 */

#include <string.h>
#include <glib.h>
#include <tapi_common.h>
#include <tapi_event.h>
#include <TapiUtility.h>
#include <ITapiCall.h>
#include <FBaseString.h>
#include <FBaseUtilStringUtil.h>
#include <FTelITelephonyCallEventListener.h>
#include <FTelITelephonyCallForwardListener.h>
#include <FTelCallManager.h>
#include <FBaseSysLog.h>
#include "FTel_CallInfoImpl.h"
#include "FTel_CallManagerImpl.h"
#include "FTel_CallManagerEvent.h"
#include "FTel_CallManagerEventArg.h"
#include "FTel_CallForwardEvent.h"
#include "FTel_CallForwardEventArg.h"
#include "FTel_TelephonyIpcProxy.h"
#include "FTel_NetworkManagerImpl.h"

using namespace std;
using namespace Tizen::App;
using namespace Tizen::Base;
using namespace Tizen::Base::Utility;


namespace Tizen { namespace Telephony
{

_CallManagerImpl::_CallManagerImpl(void)
	: __pCallEventListener(null)
	, __pCallForwardListener(null)
	, __pHandle(null)
	, __callFwdSubscriptionId(0)
	, __callFwdStatusSubscriptionId(0)
	, __callFwdReqId(INVALID_REQUEST_ID)
	, __callFwdReqNumId(INVALID_REQUEST_ID)
	, __isReqCallFwdNumInProgress(false)
	, __curCallState(CALL_STATUS_IDLE)
	, __curCallFwdState(_CALL_FORWARD_IDLE)
    , __pCallManagerEvent(null)
    , __pCallForwardEvent(null)

{
	__pCallEventType[_CALL_NOTI_VOICE_CALL_IDLE] = TAPI_NOTI_VOICE_CALL_STATUS_IDLE;
	__pCallEventType[_CALL_NOTI_VOICE_CALL_COMMUNICATING] = TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE;
	__pCallEventType[_CALL_NOTI_VOICE_CALL_RINGING] = TAPI_NOTI_VOICE_CALL_STATUS_INCOMING;
	__pCallEventType[_CALL_NOTI_VOICE_CALL_DIALING] = TAPI_NOTI_VOICE_CALL_STATUS_DIALING;
	__pCallEventType[_CALL_NOTI_VOICE_CALL_HOLDING] = TAPI_NOTI_VOICE_CALL_STATUS_HELD;
	__pCallEventType[_CALL_NOTI_VIDEO_CALL_IDLE] = TAPI_NOTI_VIDEO_CALL_STATUS_IDLE;
	__pCallEventType[_CALL_NOTI_VIDEO_CALL_COMMUNICATING] = TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE;
	__pCallEventType[_CALL_NOTI_VIDEO_CALL_RINGING] = TAPI_NOTI_VIDEO_CALL_STATUS_INCOMING;
	__pCallEventType[_CALL_NOTI_VIDEO_CALL_DIALING] = TAPI_NOTI_VIDEO_CALL_STATUS_DIALING;
	__pCallEventType[_CALL_NOTI_PEER_CALL_HELD] = TAPI_NOTI_CALL_INFO_HELD;
	__pCallEventType[_CALL_NOTI_PEER_CALL_RETRIEVED] = TAPI_NOTI_CALL_INFO_ACTIVE;

}

_CallManagerImpl::~_CallManagerImpl(void)
{
	if (__pHandle != null)
	{
		for (int i = 0; i < _CALL_NOTI_MAX; i++)
		{
			tel_deregister_noti_event(__pHandle, __pCallEventType[i]);
        }

		tel_deinit(__pHandle);
	}
}

result
_CallManagerImpl::Construct(ITelephonyCallEventListener* pListener)
{
	SysLog(NID_TEL, "The current call state is %s", GetStringOfCallState());

	result r = E_SUCCESS;
	int err = TAPI_API_SUCCESS;

	__pHandle = tel_init(null);
	SysTryCatch(NID_TEL, __pHandle != null, r = E_SYSTEM, E_SYSTEM, "[%s] A system error has occurred. Failed to initialize TApi library.", GetErrorMessage(E_SYSTEM));

	if (pListener != null)
	{
		for (int i = 0; i < _CALL_NOTI_MAX; i++)
		{
			err = tel_register_noti_event(__pHandle, __pCallEventType[i], OnCallStatusChangedCallback, this);
			SysLog(NID_TEL, "The registered call type is %s", GetStringOfCallEventType(_CALL_NOTI_VOICE_CALL_IDLE + i));

			SysTryCatch(NID_TEL, err == TAPI_API_SUCCESS, r = E_SYSTEM, E_SYSTEM,
			        "[%s] A system error has occurred. Failed to register the callback function.", GetErrorMessage(E_SYSTEM));
		}

		unique_ptr<_CallManagerEvent> pCallManagerEvent(new (std::nothrow) _CallManagerEvent);
        SysTryReturnResult(NID_TEL, pCallManagerEvent != null, E_OUT_OF_MEMORY, "Memory allocation failed.");

        r = pCallManagerEvent->Construct();
        SysTryReturnResult(NID_TEL, r != E_OUT_OF_MEMORY, r, "Propagating.");
        SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has occurred. Failed to construct CallManagerEvent.");

        pCallManagerEvent->AddListener(*pListener, true);

        __pCallManagerEvent = move(pCallManagerEvent);

	}

	r = InitializeInternalCallStatus();
	SysTryCatch(NID_TEL, r == E_SUCCESS, , E_SYSTEM, "[%s] A system error has occurred. Failed to initialize internal call state.", GetErrorMessage(E_SYSTEM));

	__pTelephonyServiceProxy = _TelephonyIpcProxy::GetInstance();

	if (__pTelephonyServiceProxy == null)
    {
        SysLog(NID_TEL, "Creating an IPC instance to connect with the Connectivity service daemon has failed.");
    }

	return r;

CATCH:
	for (int i = 0; i < _CALL_NOTI_MAX; i++)
	{
		    tel_deregister_noti_event(__pHandle, __pCallEventType[i]);
	}

	if (__pHandle != null)
	{
		tel_deinit(__pHandle);
	    __pHandle = null;
	}

	return r;
}

result
_CallManagerImpl::SetCallForwardListener(ITelephonyCallForwardListener* pListener)
{
	SysTryReturnResult(NID_TEL, __pTelephonyServiceProxy != null, E_SYSTEM, "A system error has occurred. IPC instance has not been constructed yet.");

	result r = __pTelephonyServiceProxy->HasCallForwardPrivilege();
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	if (__pCallForwardListener != null)
    {
	    __pCallForwardEvent->RemoveListener(*__pCallForwardListener);
	    __pCallForwardListener = null;
    }


	// Creates event object
    if (__pCallForwardEvent == null && pListener != null)
    {
        std::unique_ptr<_CallForwardEvent> pCallForwardEvent(new (std::nothrow) _CallForwardEvent());

         SysTryReturnResult(NID_TEL, pCallForwardEvent != null, E_OUT_OF_MEMORY, "Memory allocation failed.");

         r = pCallForwardEvent->Construct();
         SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

         __pCallForwardEvent = move(pCallForwardEvent);
    }

    // Adds listener
    if (pListener != null)
    {
        __pCallForwardListener = pListener;
        r = __pCallForwardEvent->AddListener(*__pCallForwardListener, true);
    }

    if (r != E_SUCCESS)
    {
        __pCallForwardEvent.reset(null);
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
    }

    return r;
}

result
_CallManagerImpl::RequestCallForward(const String& phoneNumber)
{
	result r = E_SUCCESS;

	switch (__curCallFwdState)
	{
	case _CALL_FORWARD_REQUESTING:
	{
		SysLogException(NID_TEL, E_IN_PROGRESS, "[%s] The previous request is in progress.", GetErrorMessage(E_IN_PROGRESS));
		return E_IN_PROGRESS;
	}

	case  _CALL_FORWARD_ON_SERVICE:
	case  _CALL_FORWARD_STOPPING:
	{
	    SysLogException(NID_TEL, E_INVALID_STATE, "[%s] The current state of the instance prohibits the execution of this operation.", GetErrorMessage(E_IN_PROGRESS));
		return E_INVALID_STATE;
	}

	default:
		break;
	}

	SysLog(NID_TEL, "The phonenumber is %ls", phoneNumber.GetPointer());

	SysTryReturnResult(NID_TEL, _NetworkManagerImpl::IsNetworkAvailable(), E_NETWORK_UNAVAILABLE,
					  "The operation failed because the device is in the offline mode.");
	SysTryReturnResult(NID_TEL,  _NetworkManagerImpl::IsServiceAvailable(), E_SERVICE_UNAVAILABLE,
					  "The operation failed because the device is out of the coverage area or in the emergency mode.");
	SysTryReturnResult(NID_TEL, !phoneNumber.IsEmpty(), E_INVALID_FORMAT, "The phone number is empty string.");
	SysTryReturnResult(NID_TEL, IsPhoneNumberValid(phoneNumber), E_INVALID_FORMAT, "The phone number is invalid format");

	r = __pTelephonyServiceProxy->RequestCallForward(this, phoneNumber);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	__curCallFwdState = _CALL_FORWARD_REQUESTING;

	return r;
}

result
_CallManagerImpl::StopCallForward(void)
{
	switch (__curCallFwdState)
	{
	case _CALL_FORWARD_STOPPING:
	{
		SysLogException(NID_TEL, E_IN_PROGRESS, "[%s] The previous request is in progress.", GetErrorMessage(E_IN_PROGRESS));
		return E_IN_PROGRESS;
	}
	case _CALL_FORWARD_IDLE:
	case _CALL_FORWARD_REQUESTING:
	{
		SysLogException(NID_TEL, E_INVALID_STATE, "[%s] The current state of the instance prohibits the execution of this operation.", GetErrorMessage(E_INVALID_STATE));
		return E_INVALID_STATE;
	}
	default:
		break;
	}
	SysTryReturnResult(NID_TEL, _NetworkManagerImpl::IsNetworkAvailable(), E_NETWORK_UNAVAILABLE,
					  "The operation has failed because the device is in the offline mode.");
	SysTryReturnResult(NID_TEL, _NetworkManagerImpl::IsServiceAvailable(), E_SERVICE_UNAVAILABLE,
	                  "The operation failed because the device is out of the coverage area or in the emergency mode.");

	result r = __pTelephonyServiceProxy->StopCallForward(this);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	__curCallFwdState = _CALL_FORWARD_STOPPING;

	return r;
}

result
_CallManagerImpl::GetCallForwardNumber(void)
{
	SysTryReturnResult(NID_TEL, __pCallForwardListener != null, E_INVALID_OPERATION,
					  "The ITelephonyCallForwardListener has not been registered yet.");
	SysTryReturnResult(NID_TEL, !__isReqCallFwdNumInProgress, E_IN_PROGRESS, "The previous request is in progress.");
	SysTryReturnResult(NID_TEL,  _NetworkManagerImpl::IsNetworkAvailable(), E_NETWORK_UNAVAILABLE,
					  "The operation has failed because the device is in the offline mode.");
	SysTryReturnResult(NID_TEL, _NetworkManagerImpl::IsServiceAvailable(), E_SERVICE_UNAVAILABLE,
					  "The operation failed because the device is out of the coverage area or in the emergency mode.");

	result r = __pTelephonyServiceProxy->GetCallForwardNumber(this);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

	__isReqCallFwdNumInProgress = true;

	return r;
}

CallType
_CallManagerImpl::GetCurrentCallType(void) const
{
	CallType type = TYPE_UNDEFINED_CALL;
	int err = TAPI_API_SUCCESS;

	err = tel_get_call_status_all(__pHandle, OnCallTypeCallback, &type);

	return type;
}

CallStatus
_CallManagerImpl::GetCurrentCallStatus(void) const
{
	return __curCallState;
}

void
_CallManagerImpl::OnCallStatusChangedCallback(TapiHandle* pHandle, const char* pNotiId, void* pData, void* pUserData)
{
	SysLog(NID_TEL, "The pNotiId value is %s", pNotiId);

	CallType callType = TYPE_UNDEFINED_CALL;
	String phoneNumber;
	int callId = 0;
	_CallManagerImpl* pCallManagerImpl = static_cast <_CallManagerImpl*>(pUserData);

	if (strcmp(pNotiId, TAPI_NOTI_VOICE_CALL_STATUS_IDLE) == 0
            || strcmp(pNotiId,TAPI_NOTI_VIDEO_CALL_STATUS_IDLE) == 0)
	{
	    TelCallStatusIdleNoti_t* pNotiData = (TelCallStatusIdleNoti_t*)pData;
	    callId = pNotiData->id;
	    pCallManagerImpl->__curCallState = CALL_STATUS_IDLE;
	}
	else
	{
	    if (strcmp(pNotiId, TAPI_NOTI_VOICE_CALL_STATUS_ACTIVE) == 0
	                || strcmp(pNotiId,TAPI_NOTI_VIDEO_CALL_STATUS_ACTIVE) == 0)
	    {
	        TelCallStatusActiveNoti_t* pNotiData = (TelCallStatusActiveNoti_t*)pData;
	        callId = pNotiData->id;
	    }
	    else if (strcmp(pNotiId,TAPI_NOTI_VOICE_CALL_STATUS_INCOMING) == 0
	            || strcmp(pNotiId,TAPI_NOTI_VIDEO_CALL_STATUS_INCOMING) == 0)
	    {
	        TelCallStatusIncomingNoti_t* pNotiData = (TelCallStatusIncomingNoti_t*)pData;
	        callId = pNotiData->id;
	    }
	    else if (strcmp(pNotiId,TAPI_NOTI_VOICE_CALL_STATUS_DIALING) == 0
	            || strcmp(pNotiId, TAPI_NOTI_VIDEO_CALL_STATUS_DIALING) == 0)
	    {
	        TelCallStatusDialingNoti_t* pNotiData = (TelCallStatusDialingNoti_t*)pData;
	        callId = pNotiData->id;
	    }
	    else if (strcmp(pNotiId,TAPI_NOTI_VOICE_CALL_STATUS_HELD) == 0)
	    {
	        TelCallStatusHeldNoti_t* pNotiData = (TelCallStatusHeldNoti_t*)pData;
	        callId = pNotiData->id;
	    }
	    else if (strcmp(pNotiId,TAPI_NOTI_CALL_INFO_HELD) == 0)
	    {
	        TelCallInfoHeldNoti_t* pNotiData = (TelCallInfoHeldNoti_t*)pData;
	        callId = pNotiData->id;
	    }
	    else if (strcmp(pNotiId,TAPI_NOTI_CALL_INFO_ACTIVE) == 0)
	    {
	        TelCallInfoActiveNoti_t* pNotiData = (TelCallInfoActiveNoti_t*)pData;
	        callId = pNotiData->id;
	    }

	    int err = tel_get_call_status_all(pHandle, OnCallStatusCallback, pCallManagerImpl);
	    SysLog(NID_TEL, "The tel_get_call_status_all() returned %d value", err);

	    phoneNumber = pCallManagerImpl->GetPhoneNumber(pHandle, callId);
		SysLog(NID_TEL, "The phonenumber is %ls", phoneNumber.GetPointer());
	}


	callType = pCallManagerImpl->GetCurrentCallType();

	if (pCallManagerImpl->__curCallState != CALL_STATUS_UNDEFINED)
	{
		unique_ptr<CallInfo> pCallInfo(null);
	    _CallInfoImpl* pCallInfoImpl = null;

	    pCallInfo.reset(new (std::nothrow) CallInfo());
	    SysTryReturnVoidResult(NID_TEL, pCallInfo != null, E_OUT_OF_MEMORY,
	    		"[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

	    pCallInfoImpl = _CallInfoImpl::GetInstance(*pCallInfo);
	    pCallInfoImpl->SetCallType(callType);
	    pCallInfoImpl->SetNumber(phoneNumber);

	    _CallManagerEventArg* pEventArg = new (std::nothrow)_CallManagerEventArg(pCallManagerImpl->__curCallState, pCallInfo.get());
	    SysTryReturnVoidResult(NID_TEL,pEventArg != null, E_OUT_OF_MEMORY,
	    		"[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

		pCallInfo.release();

		SysLog(NID_TEL, "The current call type is %d, and the phonenumber is %ls", callType, phoneNumber.GetPointer());
	    (void)pCallManagerImpl->__pCallManagerEvent->Fire(*pEventArg);
	}
}

void
_CallManagerImpl::OnTelephonyCallForwardNumberReceived(Tizen::Base::String phoneNumber, result r)
{
 	SysLog(NID_TEL, "The listener has called with result %s and phonenumber is %ls", GetErrorMessage(r), phoneNumber.GetPointer());

    _CallForwardEventArg* pEventArg = new (std::nothrow)_CallForwardEventArg(_CALL_FORWARD_EVENT_NUMBER_RECEIVED, phoneNumber, r);

    SysTryReturnVoidResult(NID_TEL,pEventArg != null, E_OUT_OF_MEMORY, "[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

    (void)__pCallForwardEvent->Fire(*pEventArg);

    __isReqCallFwdNumInProgress = false;
}

void
_CallManagerImpl::OnTelephonyCallForwardResponseReceived(Tizen::Base::String phoneNumber, result r)
{	
	SysLog(NID_TEL, "The listener has called with result %s and phonenumber is %ls", GetErrorMessage(r), phoneNumber.GetPointer());

    _CallForwardEventArg* pEventArg = new (std::nothrow)_CallForwardEventArg(_CALL_FORWARD_EVENT_RESPONSE_RECEIVED, phoneNumber, r);

    SysTryReturnVoidResult(NID_TEL, pEventArg != null, E_OUT_OF_MEMORY, "[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

    (void)__pCallForwardEvent->Fire(*pEventArg);

    if (r != E_SUCCESS)
    {
        __curCallFwdState = _CALL_FORWARD_IDLE;
    }
    else
    {
        __curCallFwdState = _CALL_FORWARD_ON_SERVICE;
    }
}

void
_CallManagerImpl::OnTelephonyCallForwardStopped(Tizen::Base::String phoneNumber, result r)
{
    SysLog(NID_TEL, "The listener has called with result %s and phonenumber is %ls", GetErrorMessage(r), phoneNumber.GetPointer());

    _CallForwardEventArg* pEventArg = new (std::nothrow)_CallForwardEventArg(_CALL_FORWARD_EVENT_STOPPED, phoneNumber, r);

    SysTryReturnVoidResult(NID_TEL, pEventArg != null, E_OUT_OF_MEMORY, "[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

    (void)__pCallForwardEvent->Fire(*pEventArg);

    if (r != E_SUCCESS)
    {
       __curCallFwdState = _CALL_FORWARD_ON_SERVICE;
    }
    else
    {
       __curCallFwdState = _CALL_FORWARD_IDLE;
    }
}

void
_CallManagerImpl::OnCallStatusCallback(TelCallStatus_t* status, void* pUserData)
{
	_CallManagerImpl* pImpl = static_cast<_CallManagerImpl*>(pUserData);

    // Priority : Ringing/Dialing > Communicating > Hold
	// ToDO : multi party call scenario 

	switch (status->CallState)
	{
	case TAPI_CALL_STATE_IDLE:
	    pImpl->__curCallState = CALL_STATUS_IDLE;
	    break;
	case TAPI_CALL_STATE_ACTIVE:
		pImpl->__curCallState = CALL_STATUS_COMMUNICATING;
		break;
	case TAPI_CALL_STATE_HELD:
	    pImpl->__curCallState = CALL_STATUS_HOLDING;
	    break;
	case TAPI_CALL_STATE_INCOMING:
	    pImpl->__curCallState = CALL_STATUS_RINGING;
	    break;
	case TAPI_CALL_STATE_DIALING:
	case TAPI_CALL_STATE_ALERT:
	    pImpl->__curCallState = CALL_STATUS_DIALING;
	    break;
	default:
	    break;
	}

	return;
}

void
_CallManagerImpl::OnCallTypeCallback(TelCallStatus_t* status, void* pUserData)
{
	CallType* pCallType = static_cast<CallType*>(pUserData);

	switch (status->CallType)
	{
	case TAPI_CALL_TYPE_VOICE:
	    *pCallType = TYPE_VOICE_CALL;
	    break;
	case TAPI_CALL_TYPE_DATA:
	    *pCallType = TYPE_VIDEO_CALL;
	    break;
	default:
	    break;
	}

	return;
}

bool
_CallManagerImpl::IsPhoneNumberValid(const String& phoneNumber) const
{
	bool isValid = true;
	int startIndex = 0;
	wchar_t chr = 0;

	if (phoneNumber.StartsWith("+", 0))
	{
		startIndex = 1;
	}

	for (int i = startIndex; i < phoneNumber.GetLength(); i++)
	{
		phoneNumber.GetCharAt(i, chr);

		if (!Character::IsDigit(chr))
		{
			isValid = false;
			break;
		}
	}
	return isValid;
}

result
_CallManagerImpl::InitializeInternalCallStatus(void)
{
	int err = TAPI_API_SUCCESS;
	result r = E_SUCCESS;

	err = tel_get_call_status_all(__pHandle, OnCallStatusCallback, this);
	SysLog(NID_TEL, "The tel_get_call_status_all() returned %d value", err);

	if (err != TAPI_API_SUCCESS)
	{
	    if (err == TAPI_API_OPERATION_FAILED)
	    {
	        __curCallState = CALL_STATUS_IDLE;
	    }
	    else
	    {
	        r = E_SYSTEM;
	    }
	}

	return r;
}

String
_CallManagerImpl::GetPhoneNumber(TapiHandle* pCallHandle, int callId) const
{
	int err = TAPI_API_SUCCESS;
	String phoneNumber;

	TelCallStatus_t callStatus = {};
	err = tel_get_call_status(pCallHandle, callId, &callStatus);
	if (err == TAPI_API_SUCCESS)
	{
	    (void)(StringUtil::Utf8ToString(callStatus.pNumber, phoneNumber));
	    SysLog(NID_TEL, "The phone number is %ls", phoneNumber.GetPointer());
	}
	else
	{
	    SysLog(NID_TEL, "The tel_get_call_status_all() returned %d value", err);
	}

	return phoneNumber;
}

const char*
_CallManagerImpl::GetStringOfCallState(void) const
{
	static const char* pStateStr[] =
	{
		"CALL_STATUS_UNDEFINED",
		"CALL_STATUS_IDLE",
		"CALL_STATUS_ACTIVE",
		"CALL_STATUS_COMMUNICATING",
		"CALL_STATUS_RINGING",
		"CALL_STATUS_DIALING",
		"CALL_STATUS_HOLDING"
	};

	return pStateStr[__curCallState + 1];
}

const char*
_CallManagerImpl::GetStringOfCallFwdState(void) const
{
	static const char* pFwdStateStr[] =
	{
		"CALL_FORWARD_IDLE",
		"CALL_FORWARD_REQUESTING",
		"CALL_FORWARD_ON_SERVICE",
		"CALL_FORWARD_STOPPING"
	};

	return pFwdStateStr[__curCallFwdState];
}

const char*
_CallManagerImpl::GetStringOfCallEventType(int type) const
{
	static const char* pEventType[] =
	{
		"Voice Call End",
		"Voice Communicating",
		"Voice Ringing",
		"Voice Dialing",
		"Voice Holding",
		"Video Call End",
		"Video Communicating",
		"Video Ringing",
		"Video Dialing",
		"Peer Held",
		"Peer Retrieved"
	};

	return pEventType[type];
}

_CallManagerImpl*
_CallManagerImpl::GetInstance(CallManager& callManager)
{
	return callManager.__pCallManagerImpl;
}

const _CallManagerImpl*
_CallManagerImpl::GetInstance(const CallManager& callManager)
{
	return callManager.__pCallManagerImpl;
}

}} // Tizen::Telephony


