//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file  FTel_CallManagerImpl.h
 * @brief This is the header file for the _CallManagerImpl class.
 *
 * This header file contains the declarations of the _CallManagerImpl Class.
 */
#ifndef _FTEL_INTERNAL_CALL_MANAGER_IMPL_H_
#define _FTEL_INTERNAL_CALL_MANAGER_IMPL_H_


#include <unique_ptr.h>
#include <tapi_type.h>
#include <TelCall.h>
#include <FBaseObject.h>
#include <FTelCallInfo.h>
#include <FTelTypes.h>


namespace Tizen { namespace Base
{
class String;
}} // Tizen::Base

namespace Tizen { namespace Telephony
{
class ITelephonyCallForwardListener;
class ITelephonyCallEventListener;
class CallManager;
class _CallManagerEvent;
class _CallForwardEvent;
class _TelephonyIpcProxy;


/**
 * @class   _CallManagerImpl
 * @brief   This class provides methods to obtain information about the current call.
 * @since 2.1
 *
 * This class provides methods to obtain information about the current call type and its status.
 *
 * The following code snippet illustrates how to obtain information about a call.
 */
class _CallManagerImpl
	: public Tizen::Base::Object
{
	/**
     *  @enum _CallForwardingState
     *  Defines the state of the call forwarding.
     */
	enum _CallForwardingState
	{
	    _CALL_FORWARD_IDLE,
	    _CALL_FORWARD_REQUESTING,
	    _CALL_FORWARD_ON_SERVICE,
	    _CALL_FORWARD_STOPPING
	};

	/**
     *  @enum _CallNotificationId
     *  Defines the call notification id
     */
	enum _CallNotificationId
	{
	    _CALL_NOTI_VOICE_CALL_IDLE,
	    _CALL_NOTI_VOICE_CALL_COMMUNICATING,
	    _CALL_NOTI_VOICE_CALL_RINGING,
	    _CALL_NOTI_VOICE_CALL_DIALING,
	    _CALL_NOTI_VOICE_CALL_HOLDING,
	    _CALL_NOTI_VIDEO_CALL_IDLE,
	    _CALL_NOTI_VIDEO_CALL_COMMUNICATING,
	    _CALL_NOTI_VIDEO_CALL_RINGING,
	    _CALL_NOTI_VIDEO_CALL_DIALING,
	    _CALL_NOTI_PEER_CALL_HELD,
	    _CALL_NOTI_PEER_CALL_RETRIEVED,
	    _CALL_NOTI_MAX

	};

public:
	/**
     * This is the default constructor for this class.
     *
     * @since 2.1
     */
	_CallManagerImpl(void);
	/**
     * This is the destructor for this class.
     *
     * @since 2.1
     */
	virtual ~_CallManagerImpl(void);

	/**
     * @see CallManager::Construct()
     */
	result Construct(ITelephonyCallEventListener* pListener = null);

	/**
     * @see CallManager::SetCallForwardListener()
     */
	result SetCallForwardListener(ITelephonyCallForwardListener* pListener);

	/**
     * @see CallManager::RequestCallForward(), OnCallForwardNumberReceivedCallback()
     *
     */
	result RequestCallForward(const Tizen::Base::String& phoneNumber);

	/**
     * @see CallManager::StopCallForward(), OnCallForwardStoppedCallback()
     */
	result StopCallForward(void);

	/**
     * @see CallManager::GetCallForwardNumber(), OnCallForwardNumberReceivedCallback()
     */
	result GetCallForwardNumber(void);

	/**
     * @see CallManager::GetCurrentCallType()
     */
	CallType GetCurrentCallType(void) const;

	/**
     * @see CallManager::GetCurrentCallStatus()
     */
	CallStatus GetCurrentCallStatus(void) const;

	void OnTelephonyCallForwardNumberReceived(Tizen::Base::String phoneNumber, result r);
	void OnTelephonyCallForwardResponseReceived(Tizen::Base::String phoneNumber, result r);
	void OnTelephonyCallForwardStopped(Tizen::Base::String phoneNumber, result r);

	/**
     * Called when the status of a call is changed.
     *
     * @param[in] pHandle   The call handle.
     * @param[in] pNotiId   The notification id.
     * @param[in] pData     The data of call status
     * @param[in] pUserData The user data passed from the callback registration function
     */
	static void OnCallStatusChangedCallback(TapiHandle *pHandle, const char* pNotiId, void* pData, void* pUserData);

	/**
     * Called when you get the information of call status
     *
     * @param[in] status    The call status
     * @param[in] pUserData The user data passed from the callback registration function
     */
	static void OnCallStatusCallback(TelCallStatus_t* pStatus, void* pUserData);

	/**
     * Called when you get the information of call type
     *
     * @param[in] status    The call status
     * @param[in] pUserData The user data passed from the callback registration function
     */
	static void OnCallTypeCallback(TelCallStatus_t* pStatus, void* pUserData);

	/**
     * Gets the Impl instance.
     *
     * @since 2.1
     * @return The pointer to _CallManagerImpl
     * @param[in] callManager   An instance of CallManager
     */
	static _CallManagerImpl* GetInstance(CallManager& callManager);

	/**
     * Gets the Impl instance.
     *
     * @since 2.1
     * @return The pointer to _CallManagerImpl
     * @param[in] callManager   An instance of CallManager
     */
	static const _CallManagerImpl* GetInstance(const CallManager& callManager);

private:
	/**
     * Initialize the call status.
     */
	result InitializeInternalCallStatus(void);

	/**
     * Checks the valid state of phone number.
     */
	bool IsPhoneNumberValid(const Tizen::Base::String& phoneNumber) const;

	/**
     * Gets the phone number.
     */
	Tizen::Base::String GetPhoneNumber(TapiHandle* pCallHandle, int callId) const;

	/**
     * Gets the string of call state.
     */
	const char* GetStringOfCallState(void) const;

	/**
     * Gets the string of call forwarding state.
     */
	const char* GetStringOfCallFwdState(void) const;

	/**
     * Gets the string of call event type.
     */
	const char* GetStringOfCallEventType(int type) const;

	/**
     * This is the copy constructor for the %_CallManagerImpl class.
     *
     * @param[in] value An instance of %_CallManagerImpl
     */
	_CallManagerImpl(const _CallManagerImpl& value);

	/**
     * This is the assignment operator.
     * Do @b not use directly.
     *
     * @param[in] rhs An instance of %_CallManagerImpl
     */
	_CallManagerImpl& operator =(const _CallManagerImpl& rhs);

private:
	ITelephonyCallEventListener* __pCallEventListener;
	ITelephonyCallForwardListener* __pCallForwardListener;

	TapiHandle* __pHandle;

	int __callFwdSubscriptionId;
	int __callFwdStatusSubscriptionId;

	int __callFwdReqId;
	int __callFwdReqNumId;

	bool __isReqCallFwdNumInProgress;

	CallStatus __curCallState;
	_CallForwardingState __curCallFwdState;

    _TelephonyIpcProxy* __pTelephonyServiceProxy;

    std::unique_ptr<_CallManagerEvent> __pCallManagerEvent;
    std::unique_ptr<_CallForwardEvent> __pCallForwardEvent;

    const char* __pCallEventType[_CALL_NOTI_MAX];

}; // _CallManagerImpl

}} // Tizen::Telephony
#endif // _FTEL_INTERNAL_CALL_MANAGER_IMPL_H_
