//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file  FTel_NetworkInfoImpl.h
 * @brief This is the header file for the _NetworkInfoImpl class.
 *
 * This header file contains the declarations of the _NetworkInfoImpl Class.
 */
#ifndef _FTEL_INTERNAL_NETWORK_INFO_IMPL_H_
#define _FTEL_INTERNAL_NETWORK_INFO_IMPL_H_


#include <FBaseObject.h>
#include <FBaseString.h>


namespace Tizen { namespace Telephony
{

class NetworkInfo;

    /**
     * @enum    NetworkType
     *
     * Defines the network type.
     *
     */
    enum _NetworkType
    {
        _NETWORK_TYPE_UNDEFINED = -1,        /**< No define type */
        _NETWORK_TYPE_GSM,                   /**< Available service is GSM  */
        _NETWORK_TYPE_GPRS,                  /**< Available service is GPRS */
        _NETWORK_TYPE_EGPRS,                 /**< Available service is EGPRS  */
        _NETWORK_TYPE_PCS1900,               /**< Available service is PCS1900 */
        _NETWORK_TYPE_UMTS,                  /**< Available service is UMTS  */
        _NETWORK_TYPE_GSM_AND_UMTS,          /**< Available service is GSM and UMTS */
        _NETWORK_TYPE_HSDPA,                 /**< Available service is HSDPA */
        _NETWORK_TYPE_IS95A,                 /**< Available service is IS95A */
        _NETWORK_TYPE_IS95B,                 /**< Available service is IS95B */
        _NETWORK_TYPE_CDMA_1X,               /**< Available service is CDMA 1X(IS-2000) */
        _NETWORK_TYPE_EVDO_REV_0,            /**< Available service is CDMA EV-DO rev0 */
        _NETWORK_TYPE_CDMA_1X_AND_EVDO_REV_0,/**< Available service is CDMA 1X(IS-2000) and EV-DO rev0 */
        _NETWORK_TYPE_EVDO_REV_A,            /**< Available service is EV-DO revA */
        _NETWORK_TYPE_CDMA_1X_AND_EVDO_REV_A,/**< Available service is CDMA 1X(IS-2000) and EV-DO revA */
        _NETWORK_TYPE_EVDV,                  /**< Available service is EV-DV */
    };

class _NetworkInfoImpl
	: public Tizen::Base::Object
{

public:
	/**
     * This is the default constructor for this class.
     *
     * @since 2.1
     */
	_NetworkInfoImpl(void);

	/**
     * This is the copy constructor for the _NetworkInfoImpl class.
     *
     * @since 2.1
     *
     * @param[in]   rhs   An instance of %_NetworkInfoImpl
     */
	_NetworkInfoImpl(const _NetworkInfoImpl& rhs);

	/**
     * This is the destructor for this class.
     *
     * @since 2.1
     */
	virtual ~_NetworkInfoImpl(void);

    /**
     * @see NetWorkInfo::GetCellId()
     */
    int GetCellId(void) const;

    /**
     * @see NetWorkInfo::GetLac()
     */
    int GetLac(void) const;

    /**
     * @see NetWorkInfo::GetMnc()
     */
	int GetMnc(void) const;

	/**
     * @see NetWorkInfo::GetMcc()
     */
	int GetMcc(void) const;

	/**
     * @see NetWorkInfo::GetPlmn()
     */
	Tizen::Base::String GetPlmn(void) const;

    /**
     * @see NetWorkInfo::GetOperatorName()
     */
    Tizen::Base::String GetOperatorName(void) const;

    /**
     * Gets the network type.
     *
     * @return  The network
     */
    _NetworkType GetNetworkType(void) const;

	/**
     * Checks whether the the specified instance of %_NetworkInfoImpl equals the current instance.
     *
     * @return   @c true if the specified instance equals the current instance, @n
     *           else @c false
     * @param[in]    rhs An instance of %_NetworkInfoImpl
     * @remarks  The method returns @c false if the specified object is not %_NetworkInfoImpl.
     */
	virtual bool Equals(const Tizen::Base::Object& rhs) const;

	/**
     * Gets the hash value of the current instance.
     *
     * @return   The hash value of the current instance
     */
	virtual int GetHashCode(void) const;

	/**
     * @see NetWorkInfo::GetRssi()
     */
	static int GetRssi(void);


    /**
     * Sets the current cell ID.
     *
     * @param[in]   cellId   The current cell ID
     */
    void SetCellId(int cellId);

    /**
     * Sets the current Location Area Code (LAC).
     *
     * @param[in]   lac   The current Location Area Code (LAC)
     */
    void SetLac(int lac);

    /**
     * Sets the Mobile Network Code (MNC) of the network.
     *
     * @param[in]   mnc   The Mobile Network Code (MNC) of the network
     */
	void SetMnc(int mnc);

	/**
     * Sets the Mobile Country Code (MCC) of the network.
     *
     * @param[in]   mcc   Mobile Country Code (MCC) of the network
     */
	void SetMcc(int mcc);

	/**
     * Sets the Public Land Mobile Network (PLMN) string.
     *
     * @param[in]   plmn   The Public Land Mobile Network (PLMN) string
     */
	void SetPlmn(const Tizen::Base::String& plmn);

    /**
     * Sets the operator name.
     *
     * @param[in]   operatorName The operator name
     */
    void SetOperatorName(const Tizen::Base::String& operatorName);

    /**
     * Sets the network type.
     *
     * @param[in]   networkType   The network type
     */
    void SetNetworkType(_NetworkType networkType);

	/**
     * Gets the Impl instance.
     *
     * @since 2.1
     * @return The pointer to _NetworkInfoImpl
     * @param[in] networkInfo   An instance of NetworkInfo
     */
	static _NetworkInfoImpl* GetInstance(NetworkInfo& networkInfo);

	/**
     * Gets the Impl instance.
     *
     * @since 2.1
     * @return The pointer to _NetworkInfoImpl
     * @param[in] networkInfo   An instance of NetworkInfo
     */
	static const _NetworkInfoImpl* GetInstance(const NetworkInfo& networkInfo);

	/**
     * This assignment operator copies the values from the specified instance of %_NetworkInfoImpl to the calling
     * instance of %_NetworkInfoImpl.
     *
     * @since 2.1
     *
     * @return      The reference of this instance
     * @param[in]   rhs   A reference to the %_NetworkInfoImpl instance to be copied
     */
	_NetworkInfoImpl& operator=(const _NetworkInfoImpl& rhs);


private:
    int __cellId;
    int __lac;
    int __mcc;
    int __mnc;
    Tizen::Base::String __plmn;
    Tizen::Base::String __operatorName;

    _NetworkType __networkType;

}; // _NetworkInfoImpl

}} // Tizen::Telephony
#endif // _FTEL_INTERNAL_NETWORK_INFO_IMPL_H_
