//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file    FTel_NetworkManagerImpl.h
 * @brief   This is the header file for the _NetworkManagerImpl class.
 *
 * This header file contains the declaration of the _NetworkManagerImpl class.
 */
#ifndef _FTEL_INTERNAL_NETWORK_MANAGER_IMPL_H_
#define _FTEL_INTERNAL_NETWORK_MANAGER_IMPL_H_

#include <unique_ptr.h>
#include <net_connection.h>
#include <telephony_network_type.h>
#include <FBaseObject.h>
#include "FTel_TelephonyUtility.h"


namespace Tizen { namespace Base
{
class String;

namespace Collection
{
class IList;
}}}

namespace Tizen { namespace Telephony
{
class ITelephonyNetworkEventListener;
class ITelephonyNetworkSettingListener;
class NetworkInfo;
class NetworkStatus;
class NetworkManager;
class _NetworkManagerEvent;
class _NetworkSettingEvent;
class _TelephonyIpcProxy;

class _NetworkManagerImpl
	: public Tizen::Base::Object
{
public:
	/**
     *	This is the default constructor for this class.
     *
     * @since 2.1
     */
	_NetworkManagerImpl(void);

	/**
     *	This is the destructor for this class.
     *
     * @since 2.1
     */
	virtual ~_NetworkManagerImpl(void);

	/**
     * @see NetworkManager::Construct()
     */
	result Construct(ITelephonyNetworkEventListener* pListener);

	/**
     * @see NetworkManager::GetNetworkStatus()
     */
	result GetNetworkStatus(NetworkStatus& networkStatus) const;

	/**
     * @see NetworkManager::GetNetworkInfo()
     */
	result GetNetworkInfo(NetworkInfo& networkInfo) const;

    /**
     * @see NetworkManager::SetNetworkSettingListener()
     */
    result SetNetworkSettingListener(ITelephonyNetworkSettingListener* pListener);

    /**
     * @see NetworkManager::GetNetworkSelectionMode()
     */
    result GetNetworkSelectionMode(void);

    /**
     * @see NetworkManager::SelectNetwork()
     */
    result SelectNetwork(const NetworkInfo& networkInfo);

    /**
     * @see NetworkManager::SelectNetwork()
     */
    result SelectNetwork(void);

    /**
     * @see NetworkManager::SearchNetwork()
     */
    result SearchNetwork(void);


    void OnTelephonyNetworkSelectionModeReceived(bool isManual, result r);

    void OnTelephonyNetworkSelectionCompleted(result r);

    void OnTelephonyNetworkSearchCompleted(Tizen::Base::String message, result r);

    /**
     * Checks the network state of the data network service.
     */
	bool IsDataAvailable(void);

    /**
     * Checks the data roaming is on or off.
     */
	bool IsRoaming(void);

    /**
     * Checks the network state of the telephony service.
     */
	static bool IsServiceAvailable(bool checkEmergency = false);

	/**
     * Checks the enabled state of flight mode.
     */
	static bool IsNetworkAvailable(void);

	/**
     * Called when the state of data service is changed.
     *
     * @param[in] type      The type of current network connection
     * @param[in] pUserData The user data passed from the callback registration function
     */
	static void OnDataServiceStateChangedCallback(connection_type_e type, void* pUserData);

	/**
     * Called when the state of call service is changed.
     *
     * @param[in] serviceState  The state of telephony service
     * @param[in] pUserData      The user data passed from the callback registration function
     */
	static void OnCallServiceStateChangedCallback(network_info_service_state_e serviceState, void* pUserData);

    /**
     * Called when the state of roaming is changed.
     *
     * @param[in] isRoaming The roaming state
     * @param[in] pUserData  The user data passed from the callback registration function
     */
	static void OnRoamingStateChangedCallback(bool isRoaming, void* pUserData);

	/**
     * Gets the Impl instance.
     *
     * @since 2.1
     * @return The pointer to _NetworkManagerImpl
     * @param[in] networkManager   An instance of NetworkManager
     */
	static _NetworkManagerImpl* GetInstance(NetworkManager& networkManager);

	/**
     * Gets the Impl instance.
     *
     * @since 2.1
     * @return The pointer to _NetworkManagerImpl
     * @param[in] networkManager   An instance of NetworkManager
     */
	static const _NetworkManagerImpl* GetInstance(const NetworkManager& networkManager);


private:
	/**
     * This is the copy constructor for the %_NetworkManagerImpl class.
     *
     * @param[in]   rhs   An instance of %_NetworkManagerImpl
     */
	_NetworkManagerImpl(const _NetworkManagerImpl& rhs);

	void OnTelephonyNetworkStatusChanged(void);

	Tizen::Base::Collection::IList* ParsingMessageN(const Tizen::Base::String& message);

	NetworkInfo* GenerateNetworkInfoN(const Tizen::Base::String& message, int& currentIndex);

	Tizen::Base::String MessageToString(const Tizen::Base::String& message, int& currentIndex);

	Tizen::Base::String MessageWithLengthToString(const Tizen::Base::String& message, int& currentIndex);

	/**
     * This is the assignment operator.
     * Do @b not use directly.
     *
     * @param[in]   rhs An instance of %_NetworkManagerImpl
     */
	_NetworkManagerImpl& operator =(const _NetworkManagerImpl& rhs);

private:
	std::unique_ptr<void, _ConnectionDeleter> __pConnectionHandle;
	std::unique_ptr<_NetworkManagerEvent> __pNetworkManagerEvent;
	std::unique_ptr<_NetworkSettingEvent> __pNetworkSettingEvent;
    _TelephonyIpcProxy* __pTelephonyServiceProxy;
    ITelephonyNetworkSettingListener* __pSettingListener;

	bool __isCallServiceAvailable;
	bool __isDataServiceAvailable;
	bool __isRoaming;

}; // _NetworkManagerImpl

}} // Tizen::Telephony
#endif // _FTEL_INTERNAL_NETWORK_MANAGER_IMPL_H_
