//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file    FTel_NetworkSettingEvent.cpp
 * @brief   This is the implementation file for the _NetworkSettingEvent Class.
 *
 * This header file contains implementation of the _NetworkSettingEvent Class.
 */

#include <unique_ptr.h>
#include <FBaseRtIEventListener.h>
#include <FBaseColArrayList.h>
#include <FBaseSysLog.h>
#include <FBaseColAllElementsDeleter.h>
#include <FTelNetworkInfo.h>
#include <FTelITelephonyNetworkSettingListener.h>
#include "FTel_NetworkInfoImpl.h"
#include "FTel_NetworkSettingEvent.h"
#include "FTel_NetworkSettingEventArg.h"

using namespace std;
using namespace Tizen::Base::Collection;

namespace Tizen { namespace Telephony
{

////////////////////////////////////////////////////////////////////////////
_NetworkSettingEvent::_NetworkSettingEvent(void)
{
}

_NetworkSettingEvent::~_NetworkSettingEvent(void)
{
}

result
_NetworkSettingEvent::Construct(void)
{
    return _Event::Initialize();
}

////////////////////////////////////////////////////////////////////////////////
/// Operation : Protected

void
_NetworkSettingEvent::FireImpl(Tizen::Base::Runtime::IEventListener& listener, const Tizen::Base::Runtime::IEventArg& arg)
{
    result r = E_SUCCESS;

    const _NetworkSettingEventArg* pArg = dynamic_cast<const _NetworkSettingEventArg*>(&arg);
    SysTryReturnVoidResult(NID_TEL, pArg != null, E_INVALID_ARG, "[%s] Invalid argument used. The arg is a null pointer.", GetErrorMessage(E_INVALID_ARG));

    ITelephonyNetworkSettingListener* pNetworkSettingListener = dynamic_cast<ITelephonyNetworkSettingListener*>(&listener);
    SysTryReturnVoidResult(NID_TEL, pNetworkSettingListener != null, E_INVALID_ARG,
            "[%s] Invalid argument used. The result of a dynamic_cast operation is null for network setting listener", GetErrorMessage(E_INVALID_ARG));


    switch(pArg->GetEventType())
    {
    case _NETWORK_EVENT_GET_SELECTION_MODE:
        pNetworkSettingListener->OnTelephonyNetworkSelectionModeReceived(pArg->GetIsManual(), pArg->GetError());
        break; // _NETWORK_EVENT_GET_SELECTION_MODE

    case _NETWORK_EVENT_SELECT_NETWORK:
        pNetworkSettingListener->OnTelephonyNetworkSelectionCompleted(pArg->GetError());
        break; // _NETWORK_EVENT_SELECT_NETWORK

    case _NETWORK_EVENT_SEARCH_COMPLETED:
		{
			ArrayList* pSrcNetworkInfoList = dynamic_cast<ArrayList*>(pArg->GetNetworkInfoList());
			SysTryCatch(NID_TEL, pSrcNetworkInfoList != null, , E_SYSTEM, "[%s] A system error has occured. Failed on dynamic casting", GetErrorMessage(E_SYSTEM));

			unique_ptr<ArrayList, AllElementsDeleter> pDescNetworkInfoList(new (std::nothrow) ArrayList());
			SysTryReturnVoidResult(NID_TEL, pDescNetworkInfoList != null, E_OUT_OF_MEMORY, "[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

			r = pDescNetworkInfoList->Construct(pSrcNetworkInfoList->GetCount());
			SysTryCatch(NID_TEL, r == E_SUCCESS, , r, "[%s] Failed to construct", GetErrorMessage(r));

			for (int i = 0; i < pSrcNetworkInfoList->GetCount(); i++)
			{
				NetworkInfo* pTempInfo =  dynamic_cast<NetworkInfo*>(pSrcNetworkInfoList->GetAt(i));
				SysTryCatch(NID_TEL, pTempInfo != null, , E_SYSTEM, "[%s] A system error has occured. Failed on dynamic casting", GetErrorMessage(E_SYSTEM));

				_NetworkInfoImpl* pTempInfoImpl = _NetworkInfoImpl::GetInstance(*pTempInfo);

				NetworkInfo* pNetworkInfo = new (std::nothrow) NetworkInfo();
				SysTryCatch(NID_TEL, pNetworkInfo != null, r = E_OUT_OF_MEMORY, E_OUT_OF_MEMORY, "[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

				_NetworkInfoImpl* pNetworkInfoImpl = _NetworkInfoImpl::GetInstance(*pNetworkInfo);

				*pNetworkInfoImpl = *pTempInfoImpl;

				r = pDescNetworkInfoList->Add(*pNetworkInfo);
				SysTryCatch(NID_TEL, r == E_SUCCESS, , r, "[%s] Failed to make NetworkInfo list", GetErrorMessage(r));
			}

			// Total Count
			SysLog(NID_TEL,"The number of network info list is %d", pDescNetworkInfoList->GetCount());

			pNetworkSettingListener->OnTelephonyNetworkSearchCompletedN(pDescNetworkInfoList.release(), pArg->GetError());
		}
        break; // _NETWORK_EVENT_SEARCH_COMPLETED

    default:
    	break; // default
    }

    return;

CATCH:

    if (r != E_SUCCESS && r != E_OUT_OF_MEMORY)
    {
        r = E_SYSTEM;
    }
    pNetworkSettingListener->OnTelephonyNetworkSearchCompletedN(null, r);
}

} } // Tizen::Telephony
