//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file        FTel_SimInfoImpl.cpp
 * @brief       This is the implementation file for _SimInfoImpl class.
 */

#include <stdlib.h>
#include <string.h>
#include <sim.h>
#include <FBaseString.h>
#include <FBaseResult.h>
#include <unique_ptr.h>
#include <FTelSimInfo.h>
#include <FBaseSysLog.h>
#include "FTel_SimInfoImpl.h"
#include "FTel_TelephonyIpcProxy.h"
#include "FTel_TelephonyUtility.h"

#include <ITapiSim.h>


using namespace std;
using namespace Tizen::App;
using namespace Tizen::Base;

namespace Tizen { namespace Telephony
{


_SimInfoImpl::_SimInfoImpl(void)
	: __isAvailable(false)
{
}

_SimInfoImpl::_SimInfoImpl(const _SimInfoImpl& rhs)
	: __isAvailable(rhs.__isAvailable)
{
}

_SimInfoImpl::~_SimInfoImpl(void)
{
}

result
_SimInfoImpl::Construct(void)
{
	int err = SIM_ERROR_NONE;
	sim_state_e simState = SIM_STATE_UNAVAILABLE;

	err = sim_get_state(&simState);
	SysLog(NID_TEL, "The return value of sim_get_state() is [0x%x] and the simState is [%d]", err, simState);

	if (err == SIM_ERROR_NONE && simState == SIM_STATE_AVAILABLE)
	{
		__isAvailable = true;
	}

	SysTryReturnResult(NID_TEL, __isAvailable, E_DEVICE_UNAVAILABLE, "The operation failed because the SIM card is not ready.");

	return E_SUCCESS;
}

int
_SimInfoImpl::GetMnc(void) const
{
	int err = SIM_ERROR_NONE;
	char* pMnc = null;
	int mnc = -1;

	SysTryReturn(NID_TEL, __isAvailable, mnc, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed because the SIM card is not ready.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	err = sim_get_mnc(&pMnc);
	SysLog(NID_TEL, "The return value of sim_get_mnc() is [0x%x] and the mnc value is [%s]", err, pMnc);
	SysTryReturn(NID_TEL, err == SIM_ERROR_NONE && pMnc != null, mnc, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	mnc = atoi(pMnc);
	free(pMnc);

	return mnc;
}

int
_SimInfoImpl::GetMcc(void) const
{
	int err = SIM_ERROR_NONE;
	char* pMcc = null;
	int mcc = -1;

	SysTryReturn(NID_TEL, __isAvailable, mcc, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed because the SIM card is not ready.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	err = sim_get_mcc(&pMcc);
	SysLog(NID_TEL, "The return value of sim_get_mcc() is [0x%x] and the mcc value is [%s]", err, pMcc);
	SysTryReturn(NID_TEL, err == SIM_ERROR_NONE && pMcc != null, mcc, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	mcc = atoi(pMcc);
	free(pMcc);

	return mcc;
}

String
_SimInfoImpl::GetSpn(void) const
{
	int err = SIM_ERROR_NONE;
	char* pSpn = null;
	String spn;

	SysTryReturn(NID_TEL, __isAvailable, spn, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed because the SIM card is not ready.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	err = sim_get_spn(&pSpn);
	SysLog(NID_TEL, "The return value of sim_get_spn() is [0x%x] and the spn value is [%s]", err, pSpn);
	SysTryReturn(NID_TEL, err == SIM_ERROR_NONE, spn, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	spn = String(pSpn);

	if (pSpn != null)
	{
		free(pSpn);
	}

	return spn;
}

String
_SimInfoImpl::GetIccId(void) const
{
	int err = SIM_ERROR_NONE;
	char* pIccId = null;
	String iccId;

	SysTryReturn(NID_TEL, __isAvailable, iccId, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed because the SIM card is not ready.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	err = sim_get_icc_id(&pIccId);
	SysSecureLog(NID_TEL, "The return value of sim_get_icc_id() is [0x%x] and the iccid value is [%s]", err, pIccId);
	SysTryReturn(NID_TEL, err == SIM_ERROR_NONE, iccId, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	iccId = String(pIccId);

	if (pIccId != null)
	{
		free(pIccId);
	}

	return iccId;
}

String
_SimInfoImpl::GetOperatorName(void) const
{
	int err = SIM_ERROR_NONE;
	char* pFullName = null;
	char* pShortName = null;
	String operatorName;

	SysTryReturn(NID_TEL, __isAvailable, operatorName, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed because the SIM card is not ready.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	err = sim_get_cphs_operator_name(&pFullName, &pShortName);
	SysSecureLog(NID_TEL, "The return value of sim_get_cphs_operator_name() is [0x%x]. full[%s] short[%s]", err, pFullName, pShortName);
	SysTryReturn(NID_TEL, err == SIM_ERROR_NONE, operatorName, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	if ((pFullName != null) && (strlen(pFullName) > 0))
	{
		operatorName = String(pFullName);
	}
	else if ((pShortName != null) && (strlen(pShortName) > 0))
	{
		operatorName = String(pShortName);
	}

	if (pFullName != null)
	{
		free(pFullName);
	}
	if (pShortName != null)
	{
		free(pShortName);
	}

	return operatorName;
}

String
_SimInfoImpl::GetPhoneNumber(void) const
{
	int err = SIM_ERROR_NONE;
	char* pPhoneNumber = null;
	String phoneNumber;

	SysTryReturn(NID_TEL, __isAvailable, phoneNumber, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed because the SIM card is not ready.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	err = sim_get_subscriber_number(&pPhoneNumber);
	SysLog(NID_TEL, "The return value of sim_get_subscriber_number() is [0x%x] and the phone number is [%s]", err, pPhoneNumber);
	SysTryReturn(NID_TEL, err == SIM_ERROR_NONE, phoneNumber, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	phoneNumber = String(pPhoneNumber);

	if (pPhoneNumber != null)
	{
		free(pPhoneNumber);
	}

	return phoneNumber;
}

String
_SimInfoImpl::GetImsi(void) const
{
	result r = E_SUCCESS;
	String imsi;

	SysTryReturn(NID_TEL, __isAvailable, imsi, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed due to a missing SIM card.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	_TelephonyIpcProxy* pTelephonyServiceProxy = _TelephonyIpcProxy::GetInstance();
	SysTryReturn(NID_TEL, pTelephonyServiceProxy != null, imsi, E_SYSTEM, "[%s] A system error has occured. IPC instance has not been constructed yet.", GetErrorMessage(E_SYSTEM));

	r = pTelephonyServiceProxy->GetImsi(imsi);
	SysTryReturn(NID_TEL, r == E_SUCCESS, imsi, r, "[%s] Failed to get the imsi.", GetErrorMessage(r));

	return imsi;
}

SimType
_SimInfoImpl::GetSimType(void) const
{
	TapiHandle* pHandle = null;
	int err = SIM_ERROR_NONE;
	TelSimCardType_t simCardType = TAPI_SIM_CARD_TYPE_UNKNOWN;
	SimType simType = SIM_TYPE_UNKNOWN;

	SysTryReturn(NID_TEL, __isAvailable, SIM_TYPE_UNKNOWN, E_DEVICE_UNAVAILABLE,
	        "[%s] The operation failed because the SIM card is not ready.", GetErrorMessage(E_DEVICE_UNAVAILABLE));

	pHandle = tel_init(null);
	SysTryReturn(NID_TEL, pHandle != null, SIM_TYPE_UNKNOWN, E_SYSTEM,
	        "[%s] A system error has occurred. Failed to initialize TAPI library.", GetErrorMessage(E_SYSTEM));

	err = tel_get_sim_type(pHandle, &simCardType);
	SysLog(NID_TEL, "The return value of tel_get_sim_type() is [0x%x] and the SIM type is [%d]", err, simCardType);

	switch(simCardType)
	{
	case TAPI_SIM_CARD_TYPE_GSM:
		simType = SIM_TYPE_GSM;
		break;
	case TAPI_SIM_CARD_TYPE_USIM:
		simType = SIM_TYPE_USIM;
		break;
	case TAPI_SIM_CARD_TYPE_RUIM:
		simType = SIM_TYPE_RUIM;
		break;
	case TAPI_SIM_CARD_TYPE_IMS:
		simType = SIM_TYPE_ISIM;
		break;
	case TAPI_SIM_CARD_TYPE_UNKNOWN:	// Fall through
	default:
		simType = SIM_TYPE_UNKNOWN;
		break;
	}

	tel_deinit(pHandle);

	return simType;
}

bool
_SimInfoImpl::IsAvailable(void) const
{
	return __isAvailable;
}

_SimInfoImpl&
_SimInfoImpl::operator =(const _SimInfoImpl& rhs)
{
	if (this != &rhs)
	{
		__isAvailable = rhs.__isAvailable;
	}

	return *this;
}

bool
_SimInfoImpl::Equals(const Tizen::Base::Object& rhs) const
{
	const _SimInfoImpl* pRhs = dynamic_cast <const _SimInfoImpl*>(&rhs);

	if (pRhs == null)
	{
		return false;
	}

	if (__isAvailable != pRhs->__isAvailable)
	{
		return false;
	}

	return true;
}

int
_SimInfoImpl::GetHashCode(void) const
{
	int hashCode = _HASH_CODE_INITIAL_VALUE;

	hashCode = _HASH_CODE_COEFFICIENT_VALUE * hashCode + (__isAvailable ? 0 : 1);

	return hashCode;
}

_SimInfoImpl*
_SimInfoImpl::GetInstance(SimInfo& simInfo)
{
	return simInfo.__pSimInfoImpl;
}

const _SimInfoImpl*
_SimInfoImpl::GetInstance(const SimInfo& simInfo)
{
	return simInfo.__pSimInfoImpl;
}

}} // Tizen::Telephony
