//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
/**
 * @file        FTel_SimStateManagerImpl.cpp
 * @brief       This is the implementation file for _SimStateManagerImpl class.
 */

// Todo: Remove unnecessary headers
#include <stdlib.h>
#include <string.h>
#include <tapi_event.h>
#include <tapi_common.h>
#include <sim.h>
#include <ITapiSim.h>
#include <TelSim.h>
#include <TapiUtility.h>

#include <unique_ptr.h>
#include <FTelITelephonySimEventListener.h>
#include <FTelISimStateManagerGetPinLockSettingResultListener.h>
#include <FTelSimStateManager.h>
#include <FBaseUtilStringUtil.h>
#include <FBaseString.h>
#include <FBaseResult.h>
#include <FBaseSysLog.h>

#include "FTel_SimInfoImpl.h"
#include "FTel_SimStateManagerImpl.h"

#include "FTel_SimEvent.h"
#include "FTel_SimEventArg.h"
#include "FTel_SimManagerEvent.h"
#include "FTel_SimManagerEventArg.h"
#include "FTel_TelephonyIpcProxy.h"
#include "FTel_TelephonyUtility.h"

using namespace std;
using namespace Tizen::System;
using namespace Tizen::App;
using namespace Tizen::Base;
using namespace Tizen::Base::Utility;
using namespace Tizen::Base::Collection;


namespace Tizen { namespace Telephony
{

_SimStateManagerImpl::_SimStateManagerImpl(void)
	: __pHandle(null)
	, __simState(SIM_STATE_UNKNOWN)
	, __isInProgress(false)
	, __pSimEvent(null)
	, __pSimManagerEvent(null)
	, __pSimEventListener(null)
	, __pSimStateManagerGetPinLockSettingResultListener(null)
{
}

_SimStateManagerImpl::~_SimStateManagerImpl(void)
{
	if (__pHandle != null)
	{
		tel_deregister_noti_event(__pHandle, TAPI_NOTI_SIM_STATUS);
		tel_deinit(__pHandle);
	}
}

result
_SimStateManagerImpl::Construct()
{
	result r = E_SUCCESS;
	int err = TAPI_API_SUCCESS;
	TelSimCardStatus_t initStatus = TAPI_SIM_STATUS_UNKNOWN;
	int cardChanged = 0;

	SysAssertf(__pHandle == null,
			"Already constructed. Calling Construct() twice or more on a same instance is not allowed for this class.");

	__pHandle = tel_init(null);
	SysTryReturnResult(NID_TEL, __pHandle != null, E_SYSTEM,
			"[%s] A system error has occurred. Failed to initialize TApi library.", GetErrorMessage(E_SYSTEM));

	err = tel_register_noti_event(__pHandle, TAPI_NOTI_SIM_STATUS, OnSimStateChangedCallback, this);
	SysTryCatch(NID_TEL, err == TAPI_API_SUCCESS, r = E_SYSTEM, E_SYSTEM,
			"[%s] A system error has occurred. Failed to register the callback function.", GetErrorMessage(E_SYSTEM));

	err = tel_get_sim_init_info(__pHandle, &initStatus, &cardChanged);
	SysLog(NID_TEL, "Initial state of SIM card is [%d:%d]", initStatus, cardChanged);
	SysTryCatch(NID_TEL, err == TAPI_API_SUCCESS, r = E_SYSTEM, E_SYSTEM,
			"[%s] A system error has occurred. Failed to get initial state of SIM.", GetErrorMessage(E_SYSTEM));

	switch (initStatus)
	{
		case TAPI_SIM_STATUS_CARD_NOT_PRESENT:			// fall through
		case TAPI_SIM_STATUS_CARD_REMOVED:
			__simState = SIM_STATE_ABSENT;
			break;

		case TAPI_SIM_STATUS_SIM_INITIALIZING:
			__simState = SIM_STATE_INITIALIZING;
			break;

		case TAPI_SIM_STATUS_SIM_INIT_COMPLETED:
			__simState = SIM_STATE_READY;
			break;

		case TAPI_SIM_STATUS_SIM_PIN_REQUIRED:
			__simState = SIM_STATE_PIN_REQUIRED;
			break;

		case TAPI_SIM_STATUS_SIM_PUK_REQUIRED:
			__simState = SIM_STATE_PUK_REQUIRED;
			break;

		case TAPI_SIM_STATUS_SIM_NCK_REQUIRED:			// fall through
		case TAPI_SIM_STATUS_SIM_NSCK_REQUIRED:			// fall through
		case TAPI_SIM_STATUS_SIM_SPCK_REQUIRED:			// fall through
		case TAPI_SIM_STATUS_SIM_CCK_REQUIRED:
			__simState = SIM_STATE_NETWORK_LOCKED;
			break;

		case TAPI_SIM_STATUS_SIM_LOCK_REQUIRED:
			__simState = SIM_STATE_SIM_LOCKED;
			break;

		case TAPI_SIM_STATUS_CARD_ERROR:				// fall through
		case TAPI_SIM_STATUS_CARD_BLOCKED:				// fall through
		case TAPI_SIM_STATUS_UNKNOWN:					// fall through
		default :
			__simState = SIM_STATE_UNKNOWN;
			break;
	} // switch

	return r;

CATCH:
	if (__pHandle != null)
	{
		tel_deinit(__pHandle);
		__pHandle = null;
	}

	return r;
}

void
_SimStateManagerImpl::OnSimStateChangedCallback(TapiHandle* pHandle, const char* pNotiId, void* pData, void* pUserData)
{
	TelSimCardStatus_t* status = (TelSimCardStatus_t*)pData;
	SysLog(NID_TEL, "TelSimCardStatus: [%d]", *status);

	_SimStateManagerImpl* pSimStateManagerImpl = static_cast<_SimStateManagerImpl*>(pUserData);

	SimState state = SIM_STATE_UNKNOWN;

	switch (*status)
	{
		case TAPI_SIM_STATUS_CARD_NOT_PRESENT:			// fall through
		case TAPI_SIM_STATUS_CARD_REMOVED:
			state = SIM_STATE_ABSENT;
			break;

		case TAPI_SIM_STATUS_SIM_INITIALIZING:
			state = SIM_STATE_INITIALIZING;
			break;

		case TAPI_SIM_STATUS_SIM_INIT_COMPLETED:
			state = SIM_STATE_READY;
			break;

		case TAPI_SIM_STATUS_SIM_PIN_REQUIRED:
			state = SIM_STATE_PIN_REQUIRED;
			break;

		case TAPI_SIM_STATUS_SIM_PUK_REQUIRED:
			state = SIM_STATE_PUK_REQUIRED;
			break;

		case TAPI_SIM_STATUS_SIM_NCK_REQUIRED:			// fall through
		case TAPI_SIM_STATUS_SIM_NSCK_REQUIRED:			// fall through
		case TAPI_SIM_STATUS_SIM_SPCK_REQUIRED:			// fall through
		case TAPI_SIM_STATUS_SIM_CCK_REQUIRED:
			state = SIM_STATE_NETWORK_LOCKED;
			break;

		case TAPI_SIM_STATUS_SIM_LOCK_REQUIRED:
			state = SIM_STATE_SIM_LOCKED;
			break;

		case TAPI_SIM_STATUS_CARD_ERROR:				// fall through
		case TAPI_SIM_STATUS_CARD_BLOCKED:				// fall through
		case TAPI_SIM_STATUS_UNKNOWN:					// fall through
		default :
			state = SIM_STATE_UNKNOWN;
			break;
	} // switch

	if (pSimStateManagerImpl->__simState != state)
	{
		pSimStateManagerImpl->__simState = state;

		if (pSimStateManagerImpl->__pSimEvent != null)
		{

			_SimEventArg* pEventArg = new (std::nothrow)_SimEventArg(_SIM_EVENT_SIM_STATE_CHANGED, state);
			SysTryReturnVoidResult(NID_TEL, pEventArg != null, E_OUT_OF_MEMORY, "[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

			(void)pSimStateManagerImpl->__pSimEvent->Fire(*pEventArg);
		}
	}
}

void
_SimStateManagerImpl::OnGetPinLockSettingCallback(TapiHandle* pHandle, int ret, void* pData, void* pUserData)
{
	result r = E_SUCCESS;
	bool isEnabled = false;

	TelSimPinOperationResult_t err = (TelSimPinOperationResult_t)ret;
	SysLog(NID_TEL, "The result of PIN operation is [%d]", err);

	if (err != TAPI_SIM_PIN_OPERATION_SUCCESS)
	{
		r = E_INVALID_STATE;
	}
	else
	{
		TelSimFacilityInfo_t* pFacilityInfo = (TelSimFacilityInfo_t*)pData;

		TelSimLockType_t lockType = pFacilityInfo->type;
		TelSimFacilityStatus_t facilityStatus = pFacilityInfo->f_status;
		SysLog(NID_TEL, "Lock type is [%d], and facility status is [%d].", lockType, facilityStatus);

		if (lockType == TAPI_SIM_LOCK_SC && facilityStatus == TAPI_SIM_FACILITY_DISABLED)
		{
			isEnabled = false;
		}
		else if (lockType == TAPI_SIM_LOCK_SC && facilityStatus == TAPI_SIM_FACILITY_ENABLED)
		{
			isEnabled = true;
		}
		else if (lockType != TAPI_SIM_LOCK_SC || facilityStatus == TAPI_SIM_FACILITY_UNKNOWN)
		{
			r = E_INVALID_STATE;
			isEnabled = false;
		}
	}

	_SimStateManagerImpl* pSimStateManagerImpl = static_cast<_SimStateManagerImpl*>(pUserData);
	pSimStateManagerImpl->__isInProgress = false;


	if (pSimStateManagerImpl->__pSimManagerEvent != null)
	{
		_SimManagerEventArg* pEventArg = new (std::nothrow)_SimManagerEventArg(_SIM_MANAGER_EVENT_PIN_LOCK_SETTING_RESULT_RECEIVED, isEnabled, r);
		SysTryReturnVoidResult(NID_TEL, pEventArg != null, E_OUT_OF_MEMORY, "[%s] Memory allocation failed.", GetErrorMessage(E_OUT_OF_MEMORY));

		(void)pSimStateManagerImpl->__pSimManagerEvent->Fire(*pEventArg);
	}
}

result
_SimStateManagerImpl::SetSimEventListener(ITelephonySimEventListener *pListener)
{
	result r = E_SUCCESS;

	if (__pSimEventListener != null)
	{
		__pSimEvent->RemoveListener(*__pSimEventListener);
		__pSimEventListener = null;
	}

	if (__pSimEvent == null && pListener != null)
	{
		std::unique_ptr<_SimEvent> pSimEvent(new (std::nothrow) _SimEvent());
		SysTryReturnResult(NID_TEL, pSimEvent != null, E_OUT_OF_MEMORY, "Memory allocation failed.");

		r = pSimEvent->Construct();
		SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

		__pSimEvent = move(pSimEvent);
	}

	if (pListener != null)
	{
		r = __pSimEvent->AddListener(*pListener, true);

		if (r == E_SUCCESS)
		{
			__pSimEventListener = pListener;
		}
	}

	if (r != E_SUCCESS)
	{
		__pSimEvent.reset(null);
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
	}

	return r;
}

result
_SimStateManagerImpl::SetSimStateManagerGetPinLockSettingResultListener(ISimStateManagerGetPinLockSettingResultListener *pListener)
{
	result r = E_SUCCESS;

	if (__pSimStateManagerGetPinLockSettingResultListener != null)
	{
		__pSimManagerEvent->RemoveListener(*__pSimStateManagerGetPinLockSettingResultListener);
		__pSimStateManagerGetPinLockSettingResultListener = null;
	}

	if (__pSimManagerEvent == null && pListener != null)
	{
		std::unique_ptr<_SimManagerEvent> pSimManagerEvent(new (std::nothrow) _SimManagerEvent());
		SysTryReturnResult(NID_TEL, pSimManagerEvent != null, E_OUT_OF_MEMORY, "Memory allocation failed.");

		r = pSimManagerEvent->Construct();
		SysTryReturnResult(NID_TEL, r == E_SUCCESS, r, "Propagating.");

		__pSimManagerEvent = move(pSimManagerEvent);
	}

	if (pListener != null)
	{
		r = __pSimManagerEvent->AddListener(*pListener, true);
		if (r == E_SUCCESS)
		{
			__pSimStateManagerGetPinLockSettingResultListener = pListener;
		}
	}

	if (r != E_SUCCESS)
	{
		__pSimManagerEvent.reset(null);
		SysLogException(NID_TEL, r, "[%s] Propagating.", GetErrorMessage(r));
	}

	return r;
}

SimState
_SimStateManagerImpl::GetSimState(void) const
{
	ClearLastResult();

	return __simState;
}

result
_SimStateManagerImpl::GetPinLockSetting(ISimStateManagerGetPinLockSettingResultListener* pListener)
{
	result r = E_SUCCESS;
	int ret;

	SysTryReturnResult(NID_TEL, pListener != null, E_INVALID_ARG, "The specified input parameter is invalid.");
	SysTryReturnResult(NID_TEL, !__isInProgress, E_IN_PROGRESS, "The previous request is in progress.");

	r = SetSimStateManagerGetPinLockSettingResultListener(pListener);
	SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has occurred. Failed to set listener.");

	ret = tel_get_sim_facility(__pHandle, TAPI_SIM_LOCK_SC, OnGetPinLockSettingCallback, this);

	SysTryReturnResult(NID_TEL, ret != TAPI_API_SIM_NOT_FOUND, E_DEVICE_UNAVAILABLE,
			"The operation failed due to a missing SIM card.");

	SysTryReturnResult(NID_TEL, ret == TAPI_API_SUCCESS || ret == TAPI_API_SIM_NOT_FOUND, E_SYSTEM,
			"A system error has occurred. Failed to get SIM facility.");

	__isInProgress = true;

	return r;
}

result
_SimStateManagerImpl::GetSimInfo(SimInfo& simInfo) const
{
	SysAssertf(__pHandle != null, "Not yet constructed. Construct() should be called before use.");

	result r = E_SUCCESS;

	_SimInfoImpl* pSimInfoImpl = _SimInfoImpl::GetInstance(simInfo);
	if (pSimInfoImpl == null)
	{
		r = simInfo.Construct();
	}
	else
	{
		r = pSimInfoImpl->Construct();
	}

	SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has occurred. Failed to construct SimInfo.");

	return r;
}

_SimStateManagerImpl*
_SimStateManagerImpl::GetInstance(SimStateManager& simStateManager)
{
	return simStateManager.__pSimStateManagerImpl;
}

const _SimStateManagerImpl*
_SimStateManagerImpl::GetInstance(const SimStateManager& simStateManager)
{
	return simStateManager.__pSimStateManagerImpl;
}

}} // Tizen::Telephony
