//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file    FTel_TelephonyIpcProxy.cpp
 * @brief   This is the implementation file for the %_TelephonyIpcProxy class.
 *
 * This file contains the implementation of the %_TelephonyIpcProxy class.
 */

#include <pthread.h>
#include <unique_ptr.h>
#include <FBaseSysLog.h>
#include <FIo_IpcClient.h>
#include <FTel_ConnectivityIpcMessages.h>
#include "FTel_TelephonyIpcProxy.h"
#include "FTel_CallManagerImpl.h"
#include "FTel_NetworkManagerImpl.h"

using namespace Tizen::Base;
using namespace Tizen::Io;

namespace Tizen { namespace Telephony
{

_TelephonyIpcProxy* _TelephonyIpcProxy::__pInstance = null;

_TelephonyIpcProxy::_TelephonyIpcProxy(void)
	: __pIpcClient(null)
    , __pNetworkManagerImplForGet(null)
	, __pNetworkManagerImplForSelect(null)
	, __pNetworkManagerImplForSearch(null)
    , __pCallManagerImplForGet(null)
	, __pCallManagerImplForStart(null)
	, __pCallManagerImplForStop(null)
{
}

_TelephonyIpcProxy::~_TelephonyIpcProxy(void)
{
	delete __pIpcClient;
}

result
_TelephonyIpcProxy::Construct(void)
{
	result r = E_SUCCESS;

	__pIpcClient = new (std::nothrow) _IpcClient();
	SysTryReturnResult(NID_TEL, __pIpcClient != null, E_OUT_OF_MEMORY, "Memory allocation failed.");

	r = __pIpcClient->Construct(TELEPHONY_CONNECTIVITY_IPC_SERVER_NAME, this);
	SysTryReturnResult(NID_TEL, r != E_OUT_OF_MEMORY, r, "Propagating.");

	if (r != E_SUCCESS)
	{
		SysLogException(NID_TEL, r, "[%s] Propagating", GetErrorMessage(r));
		delete __pIpcClient;
		__pIpcClient = null;
	}

	return r;
}

_TelephonyIpcProxy*
_TelephonyIpcProxy::GetInstance(void)
{
	static pthread_once_t onceBlock = PTHREAD_ONCE_INIT;

	if (__pInstance == null)
	{
		ClearLastResult();
		pthread_once(&onceBlock, InitSingleton);
		result r = GetLastResult();
		if(r != E_SUCCESS)
		{
			onceBlock = PTHREAD_ONCE_INIT;
		}
	}

	return __pInstance;
}

void
_TelephonyIpcProxy::InitSingleton(void)
{
	result r = E_SUCCESS;
	static _TelephonyIpcProxy instance;

	r = instance.Construct();
	SysTryReturnVoidResult(NID_TEL, r == E_SUCCESS, r, "[%s] Propagating", GetErrorMessage(r));

	__pInstance = &instance;
}

result
_TelephonyIpcProxy::GetImsi(Tizen::Base::String& imsi) const
{
	result r = E_SUCCESS;
	unsigned long ret = 0;

	std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_getImsi(&imsi, &ret));
	SysTryReturnResult(NID_TEL, pMessage!= null , E_OUT_OF_MEMORY, "Memory allocation failed.");

	r = __pIpcClient->SendRequest(*pMessage);

	SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
	SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

	SysSecureLog(NID_TEL, "The Telephony Imsi obtained through IPC is %ls.", imsi.GetPointer());

	return E_SUCCESS;
}

result
_TelephonyIpcProxy::HasSystemPrivilege(void)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_hasSystemPrivilege(&ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    return E_SUCCESS;
}

result
_TelephonyIpcProxy::GetNetworkSelectionMode(_NetworkManagerImpl* pNetworkManagerImpl)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_getNetworkSelectionMode(&ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    __pNetworkManagerImplForGet = pNetworkManagerImpl;

    return E_SUCCESS;
}

result
_TelephonyIpcProxy::SelectNetwork(_NetworkManagerImpl* pNetworkManagerImpl, const Tizen::Base::String& plmn, const int netwokrType)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_selectNetworkManual(plmn, netwokrType, &ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    __pNetworkManagerImplForSelect = pNetworkManagerImpl;

    return E_SUCCESS;
}

result
_TelephonyIpcProxy::SelectNetwork(_NetworkManagerImpl* pNetworkManagerImpl)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_selectNetworkAutomatic(&ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    __pNetworkManagerImplForSelect = pNetworkManagerImpl;

    return E_SUCCESS;

}

result
_TelephonyIpcProxy::SearchNetwork(_NetworkManagerImpl* pNetworkManagerImpl)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_searchNetwork(&ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    __pNetworkManagerImplForSearch = pNetworkManagerImpl;

    return E_SUCCESS;
}


result
_TelephonyIpcProxy::HasCallForwardPrivilege(void)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_hasCallForwardPrivilege(&ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    return E_SUCCESS;
}

result
_TelephonyIpcProxy::RequestCallForward(_CallManagerImpl* pCallManagerImpl, const String& phoneNumber)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_requestCallForward(phoneNumber, &ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    __pCallManagerImplForStart = pCallManagerImpl;

    return E_SUCCESS;
}

result
_TelephonyIpcProxy::StopCallForward(_CallManagerImpl* pCallManagerImpl)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_stopCallForward(&ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    __pCallManagerImplForStop = pCallManagerImpl;

    return E_SUCCESS;
}

result
_TelephonyIpcProxy::GetCallForwardNumber(_CallManagerImpl* pCallManagerImpl)
{
    result r = E_SUCCESS;
    unsigned long ret = 0;

    std::unique_ptr<IPC::Message> pMessage( new (std::nothrow) ConnectivityTelephonyServiceMsg_getCallForwardNumber(&ret));
    SysTryReturnResult(NID_TEL, pMessage != null , E_OUT_OF_MEMORY, "Memory allocation failed.");

    r = __pIpcClient->SendRequest(*pMessage);
    SysTryReturnResult(NID_TEL, r == E_SUCCESS, E_SYSTEM, "A system error has been occurred. Failed to send a request.");
    SysTryReturnResult(NID_TEL, ret == E_SUCCESS, ret, "Propagating.");

    __pCallManagerImplForGet = pCallManagerImpl;

    return E_SUCCESS;
}

void
_TelephonyIpcProxy::OnIpcResponseReceived(Tizen::Io::_IpcClient& client, const IPC::Message& message)
{
    SysLog(NID_TEL, "OnIpcResponseReceived");

    IPC_BEGIN_MESSAGE_MAP(_TelephonyIpcProxy, message)
    IPC_MESSAGE_HANDLER(ConnectivityTelephonyServiceMsg_onNetworkSelectionModeReceived, OnNetworkSelectionModeReceived, &client)
    IPC_MESSAGE_HANDLER(ConnectivityTelephonyServiceMsg_onNetworkSelectionCompleted, OnNetworkSelectionCompleted, &client)
    IPC_MESSAGE_HANDLER(ConnectivityTelephonyServiceMsg_onNetworkSearchCompleted, OnNetworkSearchCompleted, &client)
    IPC_MESSAGE_HANDLER(ConnectivityTelephonyServiceMsg_onCallForwardNumberReceived, OnTelephonyCallForwardNumberReceived, &client);
    IPC_MESSAGE_HANDLER(ConnectivityTelephonyServiceMsg_onCallForwardResponseReceived, OnTelephonyCallForwardResponseReceived, &client);
    IPC_MESSAGE_HANDLER(ConnectivityTelephonyServiceMsg_onCallForwardStopped, OnTelephonyCallForwardStopped, &client);
    IPC_END_MESSAGE_MAP()
}

void
_TelephonyIpcProxy::OnNetworkSelectionModeReceived(bool isManual, unsigned long res)
{
    SysLog(NID_TEL, "The listener has been called.");

    SysAssertf(__pIpcClient != null, "Not yet constructed. Construct() should be called before use.");

    __pNetworkManagerImplForGet->OnTelephonyNetworkSelectionModeReceived(isManual, res);
}

void
_TelephonyIpcProxy::OnNetworkSelectionCompleted(unsigned long res)
{
    SysLog(NID_TEL, "The listener has been called.");

    SysAssertf(__pIpcClient != null, "Not yet constructed. Construct() should be called before use.");

    __pNetworkManagerImplForSelect->OnTelephonyNetworkSelectionCompleted(res);
}

void
_TelephonyIpcProxy::OnNetworkSearchCompleted(Tizen::Base::String message, unsigned long res)
{
    SysLog(NID_TEL, "The listener has been called.");

    SysAssertf(__pIpcClient != null, "Not yet constructed. Construct() should be called before use.");

    __pNetworkManagerImplForSearch->OnTelephonyNetworkSearchCompleted(message, res);
}

void
_TelephonyIpcProxy::OnTelephonyCallForwardNumberReceived(Tizen::Base::String phoneNumber, unsigned long res)
{
    SysLog(NID_TEL, "The listener has been called.");

    SysAssertf(__pIpcClient != null, "Not yet constructed. Construct() should be called before use.");

    __pCallManagerImplForGet->OnTelephonyCallForwardNumberReceived(phoneNumber, res);
}

void
_TelephonyIpcProxy::OnTelephonyCallForwardResponseReceived(Tizen::Base::String phoneNumber, unsigned long res)
{
    SysLog(NID_TEL, "The listener has been called.");

    SysAssertf(__pIpcClient != null, "Not yet constructed. Construct() should be called before use.");

    __pCallManagerImplForStart->OnTelephonyCallForwardResponseReceived(phoneNumber, res);
}

void
_TelephonyIpcProxy::OnTelephonyCallForwardStopped(Tizen::Base::String phoneNumber, unsigned long res)
{
    SysLog(NID_TEL, "The listener has been called.");

    SysAssertf(__pIpcClient != null, "Not yet constructed. Construct() should be called before use.");

    __pCallManagerImplForStop->OnTelephonyCallForwardStopped(phoneNumber, res);
}

}} // Tizen::Telephony
