//
// Open Service Platform
// Copyright (c) 2012-2013 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file    FTel_TelephonyUtility.cpp
 * @brief   This is the implementation file for the _TelephonyUtility Class.
 *
 * This file contains implementation of the _TelephonyUtility Class.
 */


#include <FBaseSysLog.h>
#include <FBaseString.h>
#include <FBaseInteger.h>
#include "FTel_TelephonyUtility.h"



using namespace Tizen::Base;

namespace Tizen { namespace Telephony
{


_NetworkType
_TelephonyUtility::ConvertNetworkType(TelNetworkSystemType_t networkSystemType)
{
    _NetworkType networktype = _NETWORK_TYPE_UNDEFINED;
    switch(networkSystemType)
    {
    case TAPI_NETWORK_SYSTEM_NO_SRV:
        networktype = _NETWORK_TYPE_UNDEFINED;
        break;
    case TAPI_NETWORK_SYSTEM_GSM:
        networktype = _NETWORK_TYPE_GSM;
        break;
    case TAPI_NETWORK_SYSTEM_GPRS:
        networktype = _NETWORK_TYPE_GPRS;
        break;
    case TAPI_NETWORK_SYSTEM_EGPRS:
        networktype = _NETWORK_TYPE_EGPRS;
        break;
    case TAPI_NETWORK_SYSTEM_PCS1900:
        networktype = _NETWORK_TYPE_PCS1900;
        break;
    case TAPI_NETWORK_SYSTEM_UMTS:
        networktype = _NETWORK_TYPE_UMTS;
        break;
    case TAPI_NETWORK_SYSTEM_GSM_AND_UMTS:
        networktype = _NETWORK_TYPE_GSM_AND_UMTS;
        break;
    case TAPI_NETWORK_SYSTEM_HSDPA:
        networktype = _NETWORK_TYPE_HSDPA;
        break;
    case TAPI_NETWORK_SYSTEM_IS95A:
        networktype = _NETWORK_TYPE_IS95A;
        break;
    case TAPI_NETWORK_SYSTEM_IS95B:
        networktype = _NETWORK_TYPE_IS95B;
        break;
    case TAPI_NETWORK_SYSTEM_CDMA_1X:
        networktype = _NETWORK_TYPE_CDMA_1X;
        break;
    case TAPI_NETWORK_SYSTEM_EVDO_REV_0:
        networktype = _NETWORK_TYPE_EVDO_REV_0;
        break;
    case TAPI_NETWORK_SYSTEM_1X_EVDO_REV_0_HYBRID:
        networktype = _NETWORK_TYPE_CDMA_1X_AND_EVDO_REV_0;
        break;
    case TAPI_NETWORK_SYSTEM_EVDO_REV_A:
        networktype = _NETWORK_TYPE_EVDO_REV_A;
        break;
    case TAPI_NETWORK_SYSTEM_1X_EVDO_REV_A_HYBRID:
        networktype = _NETWORK_TYPE_CDMA_1X_AND_EVDO_REV_A;
        break;
    case TAPI_NETWORK_SYSTEM_EVDV:
        networktype = _NETWORK_TYPE_EVDV;
        break;
    default:
	    SysLog(NID_TEL, "The network system type is %d", networkSystemType);
		break;
    }

    return networktype;
}

TelNetworkSystemType_t
_TelephonyUtility::ConvertNetworkType(_NetworkType networkType)
{
    TelNetworkSystemType_t networkSystemType = TAPI_NETWORK_SYSTEM_NO_SRV;
    switch(networkType)
    {
    case _NETWORK_TYPE_UNDEFINED:
        networkSystemType = TAPI_NETWORK_SYSTEM_NO_SRV;
        break;
    case _NETWORK_TYPE_GSM:
        networkSystemType = TAPI_NETWORK_SYSTEM_GSM;
        break;
    case _NETWORK_TYPE_GPRS:
        networkSystemType = TAPI_NETWORK_SYSTEM_GPRS;
        break;
    case _NETWORK_TYPE_EGPRS:
        networkSystemType = TAPI_NETWORK_SYSTEM_EGPRS;
        break;
    case _NETWORK_TYPE_PCS1900:
        networkSystemType = TAPI_NETWORK_SYSTEM_PCS1900;
        break;
    case _NETWORK_TYPE_UMTS:
        networkSystemType = TAPI_NETWORK_SYSTEM_UMTS;
        break;
    case _NETWORK_TYPE_GSM_AND_UMTS:
        networkSystemType = TAPI_NETWORK_SYSTEM_GSM_AND_UMTS;
        break;
    case _NETWORK_TYPE_HSDPA:
        networkSystemType = TAPI_NETWORK_SYSTEM_HSDPA;
        break;
    case _NETWORK_TYPE_IS95A:
        networkSystemType = TAPI_NETWORK_SYSTEM_IS95A;
        break;
    case _NETWORK_TYPE_IS95B:
        networkSystemType = TAPI_NETWORK_SYSTEM_IS95B;
        break;
    case _NETWORK_TYPE_CDMA_1X:
        networkSystemType = TAPI_NETWORK_SYSTEM_CDMA_1X;
        break;
    case _NETWORK_TYPE_EVDO_REV_0:
        networkSystemType = TAPI_NETWORK_SYSTEM_EVDO_REV_0;
        break;
    case _NETWORK_TYPE_CDMA_1X_AND_EVDO_REV_0:
        networkSystemType = TAPI_NETWORK_SYSTEM_1X_EVDO_REV_0_HYBRID;
        break;
    case _NETWORK_TYPE_EVDO_REV_A:
        networkSystemType = TAPI_NETWORK_SYSTEM_EVDO_REV_A;
        break;
    case _NETWORK_TYPE_CDMA_1X_AND_EVDO_REV_A:
        networkSystemType = TAPI_NETWORK_SYSTEM_1X_EVDO_REV_A_HYBRID;
        break;
    case _NETWORK_TYPE_EVDV:
        networkSystemType = TAPI_NETWORK_SYSTEM_EVDV;
        break;
    default:
	    SysLog(NID_TEL, "The network type is %d", networkType);
		break;
    }

    return networkSystemType;
}

result
_TelephonyUtility::ConvertErrorType(int error)
{
    result r = E_SUCCESS;

    if (error == 0)
    {
        r = E_SUCCESS;
    }
    else if (error == -1)
    {
        r = E_NOT_RESPONDING;
    }
    else
    {
        r = E_SYSTEM;
    }

    return r;
}

void
_TelephonyUtility::GetMccMncFromPlmn(const String& plmn, int& mcc, int& mnc)
{
    int currentIndex = 0;
    const int MCC_LENGTH = 3;
    String temp;

    plmn.SubString(currentIndex, MCC_LENGTH, temp);
    currentIndex +=  MCC_LENGTH;
    Integer::Parse(temp, static_cast<int&>(mcc));
    SysLog(NID_TEL, "The temp value is %ls", temp.GetPointer());
    SysLog(NID_TEL, "The mcc value is %d", mcc);

    temp.Clear();

    plmn.SubString(currentIndex, plmn.GetLength()-MCC_LENGTH, temp);
    Integer::Parse(temp, static_cast<int&>(mnc));
    SysLog(NID_TEL, "The temp value is %ls", temp.GetPointer());
    SysLog(NID_TEL, "The mnc value is %d", mnc);
}

} } // Tizen::Telephony
